/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#include "AutomapActions.h"

// Reaper API functions (for non-NULL function pointer check)
void* (*EnumProjects)(int idx, char *projfn, int projfnlen);
void (*Main_OnCommand)(int command, int flag);
double (*CSurf_OnVolumeChangeEx)(MediaTrack *trackid, double volume, bool relative, bool allowGang);
double (*CSurf_OnPanChangeEx)(MediaTrack *trackid, double pan, bool relative, bool allowGang);
void *(*GetSetTrackSendInfo)(MediaTrack *tr, int category, int sendidx, const char *parmname, void *setNewValue);
void *(*GetSetMediaTrackInfo)(MediaTrack *tr, const char *parmname, void *setNewValue);
bool (*kbd_RunCommandThroughHooks)(KbdSectionInfo *section, int *actionCommandID,int *val, int *valhw, int *relmode, HWND hwnd); //actioncommandID may get modified
int (*KBD_OnMainActionEx)(int cmd, int val, int valhw, int relmode, HWND hwnd, ReaProject* proj);
const char* (*GetExePath)();
int (*ShowMessageBox)(const char* msg, const char* title, int type);
HWND (*GetMainHwnd)();
int (*plugin_register)(const char* name, void* infostruct);
//int (*GetTrackAutomationMode)(MediaTrack* tr);
//void (*SetTrackAutomationMode)(MediaTrack* tr, int mode);
//void (*CSurf_SetAutoMode)(int mode, IReaperControlSurface* ignoresurf);
void* (*MIDIEditor_GetActive)();
bool (*MIDIEditor_OnCommand)(void* midieditor, int command_id);

WDL_DLGRET setFnAliasDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
    {
        case WM_INITDIALOG:
		{
			const char* args = (const char*)lParam;
			strcpy(AutomapRegisterer::_argsBuffer, args);

			int type, slot;
			if(sscanf(args, "%d %d", &type, &slot) == 2)
			{
				const char* alias = NULL;
				switch(type)
				{
					case 0:		alias = AutomapCSurf::getFnButAlias(slot);	break;
					case 1:		alias = AutomapCSurf::getFnPotAlias(slot);	break;
					case 2:		alias = AutomapCSurf::getFnEncAlias(slot);	break;
					default:
						return 0;
					break;
				}

				SetDlgItemText(hwnd, IDC_RENAME_FNCONTROL, alias);
#ifdef _WIN32
				SendMessage(hwnd, WM_NEXTDLGCTL, (WPARAM)GetDlgItem(hwnd, IDC_RENAME_FNCONTROL), TRUE);
#endif
			}
			return 0;
		}

		case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case IDOK:
				{
					char alias[BUFFER_SIZE];
					GetDlgItemText(hwnd,IDC_RENAME_FNCONTROL,alias,BUFFER_SIZE);

					int type, slot;
					if(sscanf(AutomapRegisterer::_argsBuffer, "%d %d", &type, &slot) == 2)
					{
						switch(type)
						{
							case 0:		AutomapCSurf::setFnButAlias(slot, alias);	break;
							case 1:		AutomapCSurf::setFnPotAlias(slot, alias);	break;
							case 2:		AutomapCSurf::setFnEncAlias(slot, alias);	break;
							default:
							break;
						}
						AutomapCSurf::saveConfigFile();
					}

					EndDialog(hwnd,0);
					return 0;
				}

				case IDCANCEL:
				{
					EndDialog(hwnd,0);
					return 0;
				}
			}
	}

	return 0;
}

ReaperAction::ReaperAction()
{
}

ReaperAction::ReaperAction(const char* description, void (*process)(const char*), unsigned short key, char virtualKey, char cmdKey, const char* processArgs)
{
	_process = process;
	strcpy(_processArgs, processArgs);
	strcpy(_description, description);
	_gaccel.desc = _description;
	_gaccel.accel.key = key;
	_gaccel.accel.fVirt = virtualKey;
	_gaccel.accel.cmd = cmdKey;
}

ReaperAction::~ReaperAction()
{
}

vector<ReaperAction*> AutomapRegisterer::_actions;
char AutomapRegisterer::_argsBuffer[BUFFER_SIZE];
bool AutomapRegisterer::_areActionsRegistered = false;

AutomapRegisterer::AutomapRegisterer()
{
}

AutomapRegisterer::~AutomapRegisterer()
{
}

ReaperAction* AutomapRegisterer::getAction(int index)
{
	return _actions[index];
}

unsigned int AutomapRegisterer::getNbActions()
{
	return (unsigned int)(_actions.size());
}

void AutomapRegisterer::toggleMcpMode(const char* args)
{
	AutomapCSurf::toggleMcpMode();
}

void AutomapRegisterer::toggleFfwRwdMode(const char* args)
{
	AutomapCSurf::toggleFfwRwdMode();
}

void AutomapRegisterer::increaseZoomFactor(const char* args)
{
	AutomapCSurf::increaseZoomFactor();
}

void AutomapRegisterer::decreaseZoomFactor(const char* args)
{
	AutomapCSurf::decreaseZoomFactor();
}

void AutomapRegisterer::increaseFfwRwdRange(const char* args)
{
	AutomapCSurf::increaseFfwRwdRange();
}

void AutomapRegisterer::decreaseFfwRwdRange(const char* args)
{
	AutomapCSurf::decreaseFfwRwdRange();
}

void AutomapRegisterer::increaseSpeedDialFactor(const char* args)
{
	AutomapCSurf::increaseSpeedDialFactor();
}

void AutomapRegisterer::decreaseSpeedDialFactor(const char* args)
{
	AutomapCSurf::decreaseSpeedDialFactor();
}

void AutomapRegisterer::setFnMidiChannel(const char* args)
{
	int chan = atoi(args);
	AutomapCSurf::setFnMidiChannel(chan);
	AutomapCSurf::saveConfigFile();
}

void AutomapRegisterer::setFnAlias(const char* args)
{
	HWND hwndParent = GetMainHwnd();
	DialogBoxParam(g_hInst, MAKEINTRESOURCE(IDD_AUTOMAP_RENAME_FNCONTROL), hwndParent, setFnAliasDlgProc, (LPARAM)args);
}

void AutomapRegisterer::resetFnAliases(const char* args)
{
	AutomapCSurf::resetFnAliases();
	AutomapCSurf::saveConfigFile();
}

bool AutomapRegisterer::hookCommandProc(int command, int flag)
{
	for(unsigned int i=0; i<getNbActions(); i++)
	{
		if (command == getAction(i)->_gaccel.accel.cmd)
		{
			getAction(i)->_process(getAction(i)->_processArgs);
			return true;
		}
	}

	return false;
}

//bool AutomapRegisterer::registerActions(reaper_plugin_info_t *rec)
//{
//	_actions.push_back(new ReaperAction("Novation Automap: toggle track/mixer view", &toggleMcpMode, 'V', FALT, 0));
//	_actions.push_back(new ReaperAction("Novation Automap: change ffw/rwd mode", &toggleFfwRwdMode, 'N', FALT, 0));
//	_actions.push_back(new ReaperAction("Novation Automap: increase zoom factor", &increaseZoomFactor));
//	_actions.push_back(new ReaperAction("Novation Automap: decrease zoom factor", &decreaseZoomFactor));
//	_actions.push_back(new ReaperAction("Novation Automap: increase ffw/rwd range factor", &increaseFfwRwdRange));
//	_actions.push_back(new ReaperAction("Novation Automap: decrease ffw/rwd range factor", &decreaseFfwRwdRange));
//	for(int i=0; i<16; i++)
//	{
//		char name[BUFFER_SIZE];
//		char args[BUFFER_SIZE];
//		sprintf(name, "Novation Automap: set Fn* MIDI channel: %02d", i+1);
//		sprintf(args, "%d", i);
//		_actions.push_back(new ReaperAction(name, &setFnMidiChannel, 0, 0, 0, args));
//	}
//
//	for(int i=0; i<32; i++)
//	{
//		char name[BUFFER_SIZE];
//		char args[BUFFER_SIZE];
//		sprintf(name, "Novation Automap: set alias for Fn%02d", i+1);
//		sprintf(args, "0 %d", i);
//		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
//	}
//
//	for(int i=0; i<16; i++)
//	{
//		char name[BUFFER_SIZE];
//		char args[BUFFER_SIZE];
//		sprintf(name, "Novation Automap: set alias for FnPot%02d", i+1);
//		sprintf(args, "1 %d", i);
//		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
//	}
//
//	for(int i=0; i<8; i++)
//	{
//		char name[BUFFER_SIZE];
//		char args[BUFFER_SIZE];
//		sprintf(name, "Novation Automap: set alias for FnEnc%02d", i+1);
//		sprintf(args, "2 %d", i);
//		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
//	}
//
//	_actions.push_back(new ReaperAction("Novation Automap: clear all Fn* aliases", &resetFnAliases));
//
//	char commandId[512];
//	for(unsigned int i=0; i<getNbActions(); i++)
//	{
//		sprintf(commandId, "AUTOMAP%d", i);
//		getAction(i)->_gaccel.accel.cmd = rec->Register("command_id",(void*)commandId);
//		if(!getAction(i)->_gaccel.accel.cmd)
//			return false;
//		rec->Register("gaccel",&(getAction(i)->_gaccel));
//	}
//	rec->Register("hookcommand",(void*)hookCommandProc);
//
//	return true;
//}

bool AutomapRegisterer::registerActions()
{
	if(_areActionsRegistered)
		return true;

	_actions.push_back(new ReaperAction("Novation Automap: toggle track/mixer view", &toggleMcpMode, 'V', FALT, 0));
	_actions.push_back(new ReaperAction("Novation Automap: change ffw/rwd mode", &toggleFfwRwdMode, 'N', FALT, 0));
	_actions.push_back(new ReaperAction("Novation Automap: increase zoom factor", &increaseZoomFactor));
	_actions.push_back(new ReaperAction("Novation Automap: decrease zoom factor", &decreaseZoomFactor));
	_actions.push_back(new ReaperAction("Novation Automap: increase ffw/rwd range factor", &increaseFfwRwdRange));
	_actions.push_back(new ReaperAction("Novation Automap: decrease ffw/rwd range factor", &decreaseFfwRwdRange));
	_actions.push_back(new ReaperAction("Novation Automap: increase Speed Dial factor", &increaseSpeedDialFactor));
	_actions.push_back(new ReaperAction("Novation Automap: decrease Speed Dial factor", &decreaseSpeedDialFactor));

	for(int i=0; i<16; i++)
	{
		char name[BUFFER_SIZE];
		char args[BUFFER_SIZE];
		sprintf(name, "Novation Automap: set Fn* MIDI channel: %02d", i+1);
		sprintf(args, "%d", i);
		_actions.push_back(new ReaperAction(name, &setFnMidiChannel, 0, 0, 0, args));
	}

	for(int i=0; i<32; i++)
	{
		char name[BUFFER_SIZE];
		char args[BUFFER_SIZE];
		sprintf(name, "Novation Automap: set alias for Fn%02d", i+1);
		sprintf(args, "0 %d", i);
		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
	}

	for(int i=0; i<16; i++)
	{
		char name[BUFFER_SIZE];
		char args[BUFFER_SIZE];
		sprintf(name, "Novation Automap: set alias for FnPot%02d", i+1);
		sprintf(args, "1 %d", i);
		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
	}

	for(int i=0; i<8; i++)
	{
		char name[BUFFER_SIZE];
		char args[BUFFER_SIZE];
		sprintf(name, "Novation Automap: set alias for FnEnc%02d", i+1);
		sprintf(args, "2 %d", i);
		_actions.push_back(new ReaperAction(name, &setFnAlias, 0, 0, 0, args));
	}

	_actions.push_back(new ReaperAction("Novation Automap: clear all Fn* aliases", &resetFnAliases));

	char commandId[512];
	for(unsigned int i=0; i<getNbActions(); i++)
	{
		sprintf(commandId, "AUTOMAP%d", i);
		getAction(i)->_gaccel.accel.cmd = plugin_register("command_id",(void*)commandId);
		if(!getAction(i)->_gaccel.accel.cmd)
			return false;
		plugin_register("gaccel",&(getAction(i)->_gaccel));
	}
	plugin_register("hookcommand",(void*)hookCommandProc);

	_areActionsRegistered = true;

	return true;
}

bool AutomapRegisterer::unregisterActions()
{
	plugin_register("-hookcommand",(void*)hookCommandProc);

	char commandId[512];
	for(unsigned int i=0; i<getNbActions(); i++)
	{
		sprintf(commandId, "AUTOMAP%d", i);
		plugin_register("-gaccel",&(getAction(i)->_gaccel));
		plugin_register("-command_id",(void*)commandId);
	}

	clearActions();

	_areActionsRegistered = false;

	return true;
}

void AutomapRegisterer::clearActions()
{
	for(unsigned int i=0; i<_actions.size(); i++)
	{
		if(_actions[i] != NULL)
		{
			delete _actions[i];
			_actions[i] = NULL;
		}
	}
	_actions.clear();
}

void AutomapRegisterer::createSubMenu(HWND hWnd)
{
	#ifdef _WIN32
		int menuIdx = 6;
	#else	// macOS
		int menuIdx = 6;
	#endif

	HMENU hMenu = GetSubMenu(GetMenu(hWnd), menuIdx);
	MENUITEMINFO mi={sizeof(MENUITEMINFO),};
	mi.fMask = MIIM_TYPE | MIIM_ID;
	mi.fType = MFT_STRING;

	char tmp[512];
	for(unsigned int i=0; i<getNbActions(); i++)
	{
		strcpy(tmp, getAction(i)->_gaccel.desc);
		mi.dwTypeData = tmp;
		mi.wID = getAction(i)->_gaccel.accel.cmd;
		InsertMenuItem(hMenu, 1, TRUE, &mi);
	}
}

bool AutomapRegisterer::registerSurface(reaper_plugin_info_t *rec, HWND *hWnd)
{
	if(rec == NULL)
	{
		MessageBox(NULL, "Could not add Novation Automap Control Surface", "Error!", MB_OK|MB_ICONERROR);
		return false;
	}

	int errcnt = 0;
	#define AUTOMAP_APICHECK(x) if (!((*((void **)&(x)) = (void *)rec->GetFunc(#x)))) errcnt++;
	AUTOMAP_APICHECK(EnumProjects)
	AUTOMAP_APICHECK(Main_OnCommand)
	AUTOMAP_APICHECK(CSurf_OnVolumeChangeEx)
	AUTOMAP_APICHECK(CSurf_OnPanChangeEx)
	AUTOMAP_APICHECK(GetSetTrackSendInfo)
	AUTOMAP_APICHECK(GetSetMediaTrackInfo)
	AUTOMAP_APICHECK(kbd_RunCommandThroughHooks)
	AUTOMAP_APICHECK(KBD_OnMainActionEx)
	AUTOMAP_APICHECK(GetExePath)
	AUTOMAP_APICHECK(ShowMessageBox)
	AUTOMAP_APICHECK(GetMainHwnd)
	AUTOMAP_APICHECK(plugin_register)
	//AUTOMAP_APICHECK(GetTrackAutomationMode)
	//AUTOMAP_APICHECK(SetTrackAutomationMode)
	//AUTOMAP_APICHECK(CSurf_SetAutoMode)
	AUTOMAP_APICHECK(MIDIEditor_GetActive)
	AUTOMAP_APICHECK(MIDIEditor_OnCommand)

	if(errcnt)
	{
		MessageBox(NULL, "Could not add Novation Automap Control Surface", "Error!", MB_OK|MB_ICONERROR);
		return false;
	}

	//// NEVER return false after rec->Register() or Reaper will crash badly!!!
	//if(!registerActions(rec))
	//{
	//	MessageBox(NULL, "Could not register 'Novation Automap: ...' actions", "Warning", MB_OK|MB_ICONWARNING);
	//	clearActions();
	//	return true;
	//}

	//! \note No submenu for now (useless)
	//if(hWnd == NULL)
	//	MessageBox(NULL, "Could not add Novation Automap submenu", "Error!", MB_OK|MB_ICONERROR);
	//createSubMenu(*hWnd);

	// NEVER return false after rec->Register("csurf",...) or Reaper will crash badly!!!
	if(!rec->Register("csurf",&csurf_automap_reg))
	{
		MessageBox(NULL, "Could not register Novation Automap CSurf", "Warning", MB_OK|MB_ICONWARNING);
		clearActions();
	}

	return true;
}

