/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#ifndef __AUTOMAP_ACTIONS_H__
#define __AUTOMAP_ACTIONS_H__

#include <vector>
using namespace std;

//#include "csurf.h"
#include "AutomapCSurf.h"

extern reaper_csurf_reg_t csurf_automap_reg;

//! \brief Dialog proc for 'set Fn* alias' action
WDL_DLGRET setFnAliasDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);

//! \brief Reaper action structure
class ReaperAction
{
	private:
		ReaperAction();						//!< Constructor (forbidden access)

	public:
		gaccel_register_t _gaccel;			//!< Action registerer
		void (*_process)(const char*);		//!< Pointer to the function called by action
		char _processArgs[BUFFER_SIZE];		//!< Arguments of process() function
		char _description[BUFFER_SIZE];		//!< Description of action

		ReaperAction(const char* description, void (*process)(const char*), unsigned short key = 0, char virtualKey = 0, char cmdKey = 0, const char* processArgs = "");	//!< Constructor
		~ReaperAction();					//!< Destructor
};

//! \brief Automap registration container
//! \note Static members/methods only!
//! \bug Reaper: NEVER return 0 in REAPER_PLUGIN_ENTRYPOINT() after rec->Register() or Reaper will crash badly! (bug?)
class AutomapRegisterer
{
	friend WDL_DLGRET setFnAliasDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam);

	private:
		static vector<ReaperAction*> _actions;			//!< Set of actions
		static char _argsBuffer[BUFFER_SIZE];			//!< Function arguments (temp)
		static bool _areActionsRegistered;

		AutomapRegisterer();							//!< Constructor (forbidden access: singleton)
		~AutomapRegisterer();							//!< Destructor (forbidden access: singleton)

		static ReaperAction* getAction(int index);		//!< Return ptr to action at index
		static unsigned int getNbActions();				//!< Return number of actions

		static void toggleMcpMode(const char* args = NULL);				//!< Toggle Track/Mixer view 
		static void toggleFfwRwdMode(const char* args = NULL);			//!< Change FFW/RWD mode 
		static void increaseZoomFactor(const char* args = NULL);		//!< Increase zoom factor
		static void decreaseZoomFactor(const char* args = NULL);		//!< Decrease zoom factor
		static void increaseFfwRwdRange(const char* args = NULL);		//!< Increase FFW/RWD range ('move cursor' mode)
		static void decreaseFfwRwdRange(const char* args = NULL);		//!< Decrease FFW/RWD range ('move cursor' mode)
		static void setFnMidiChannel(const char* args);					//!< Change Fn assignable controls MIDI channel
		static void setFnAlias(const char* args);						//!< Set aliases for Fn assignable controls
		static void resetFnAliases(const char* args = NULL);			//!< Reset aliases of all Fn assignable controls
		static void increaseSpeedDialFactor(const char* args = NULL);	//!< Increase Speed Dial factor
		static void decreaseSpeedDialFactor(const char* args = NULL);	//!< Decrease Speed Dial factor

		static bool hookCommandProc(int command, int flag);			//!< Hook command method

		//static bool registerActions(reaper_plugin_info_t *rec);	//!< Register actions to host
		static void createSubMenu(HWND hWnd);						//!< Create submenu in host (unused)

	public:
		static bool registerSurface(reaper_plugin_info_t *rec, HWND *hWnd = NULL);	//!< Call this to register (static)
		static void clearActions();													//!< Clear actions
		static bool registerActions();												//!< Register actions to host
		static bool unregisterActions();											//!< Unregister actions from host
};

#endif
