/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#ifndef __AUTOMAP_CSURF_H__
#define __AUTOMAP_CSURF_H__

#include <stdarg.h>

#include <vector>
#include <deque>
#include <set>
#include <map>
using namespace std;

#include "csurf.h"
#include "AutomapSDK/include/Automap.h"

#include "AutomapDevice.h"
#include "AutomapExtraMappings.h"

class ReaProject;
class TrackEnvelope;
//extern void* (*EnumProjects)(int idx, char *projfn, int projfnlen);
//extern void (*Main_OnCommand)(int command, int flag);
//extern double (*CSurf_OnVolumeChangeEx)(MediaTrack *trackid, double volume, bool relative, bool allowGang);
//extern double (*CSurf_OnPanChangeEx)(MediaTrack *trackid, double pan, bool relative, bool allowGang);
//extern void *(*GetSetTrackSendInfo)(MediaTrack *tr, int category, int sendidx, const char *parmname, void *setNewValue);
//extern void *(*GetSetMediaTrackInfo)(MediaTrack *tr, const char *parmname, void *setNewValue);
//extern bool (*kbd_RunCommandThroughHooks)(KbdSectionInfo *section, int *actionCommandID,int *val, int *valhw, int *relmode, HWND hwnd);
//extern int (*KBD_OnMainActionEx)(int cmd, int val, int valhw, int relmode, HWND hwnd, ReaProject* proj);
//extern const char* (*GetExePath)();
//extern int (*ShowMessageBox)(const char* msg, const char* title, int type);
//extern HWND (*GetMainHwnd)();
//#include "reaper_extension_sdk/jmde/reaper_plugin_functions.h"
#include "reaper_plugin_functions.h"

#define ID_MIDI_VZOOM_IN					40111
#define ID_MIDI_VZOOM_OUT					40112
#define ID_MIDI_HZOOM_OUT					1011
#define ID_MIDI_HZOOM_IN					1012
#define ID_MIDI_SCROLLVIEW_UP				40138
#define ID_MIDI_SCROLLVIEW_DOWN				40139
#define ID_MIDI_SCROLLVIEW_LEFT				40140
#define ID_MIDI_SCROLLVIEW_RIGHT			40141
#define ID_MIDI_MOVE_LEFT					40047
#define ID_MIDI_MOVE_RIGHT					40048
#define ID_MIDI_MOVE_UP						40049
#define ID_MIDI_MOVE_DOWN					40050
#define ID_MIDI_GOTO_PLAY					40150
#define ID_MIDI_GOTO_EDIT					40151
#define ID_MIDI_INSERTNOTE_MOUSECURSOR		40001
#define ID_MIDI_INSERTNOTE_EDITCURSOR		40051
#define ID_MIDI_NOTEPROPERTIES				40004
#define ID_MIDI_VELOCITY_INC1				40462
#define ID_MIDI_VELOCITY_INC10				40463
#define ID_MIDI_VELOCITY_DEC1				40464
#define ID_MIDI_VELOCITY_DEC10				40465
#define ID_MIDI_VIEW_PIANOROLL				40042
#define ID_MIDI_VIEW_DRUMMAP				40043
#define ID_MIDI_VIEW_EVENTLIST				40056
#define ID_MIDI_TOGGLESNAP					1014
#define ID_MIDI_TOGGLEGRID					1017
#define ID_MIDI_GRID_MULTSIZE2				40210
#define ID_MIDI_GRID_MULTSIZE3				40211
#define ID_MIDI_GRID_DIVSIZE2				40212
#define ID_MIDI_GRID_DIVSIZE3				40213
#define ID_MIDI_GRID_SETSIZE1_48			40189
#define ID_MIDI_GRID_SETSIZE1_32			40190
#define ID_MIDI_GRID_SETSIZE1_24			40191
#define ID_MIDI_GRID_SETSIZE1_16			40192
#define ID_MIDI_GRID_SETSIZE1_12			40193
#define ID_MIDI_GRID_SETSIZE1_11			40194
#define ID_MIDI_GRID_SETSIZE1_10			40195
#define ID_MIDI_GRID_SETSIZE1_9				40196
#define ID_MIDI_GRID_SETSIZE1_8				40197
#define ID_MIDI_GRID_SETSIZE1_7				40198
#define ID_MIDI_GRID_SETSIZE1_6				40199
#define ID_MIDI_GRID_SETSIZE1_5				40200
#define ID_MIDI_GRID_SETSIZE1_4				40201
#define ID_MIDI_GRID_SETSIZE1_3				40202
#define ID_MIDI_GRID_SETSIZE1_2				40203
#define ID_MIDI_GRID_SETSIZE1				40204
#define ID_MIDI_GRID_SETSIZE2				40205
#define ID_MIDI_GRID_SETSIZE3				40206
#define ID_MIDI_GRID_SETSIZE4				40207
#define ID_MIDI_GRID_SETSIZE5				40208
#define ID_MIDI_GRID_SETSIZE6				40209
#define ID_MIDI_NEXT_SECONDARY				40500
#define ID_MIDI_TRANSPOSE_INC1				40177
#define ID_MIDI_TRANSPOSE_DEC1				40178
#define ID_MIDI_TRANSPOSE_INC12				40179
#define ID_MIDI_TRANSPOSE_DEC12				40180
#define ID_MIDI_HUMANIZE					40457
#define ID_MIDI_FIT_CONTENT					40466
#define ID_MIDI_FIT_LOOP					40468
#define ID_MIDI_DELETENOTE					40002
#define ID_MIDI_UNSELECT_ALL				40214

#define ID_MIDI_NOTE_LENGTHEN				40446
#define ID_MIDI_NOTE_SHORTEN				40447
#define ID_MIDI_CCONLY_VSCROLL				40432
#define ID_MIDI_CCONLY_HSCROLL				40433

#define ID_MAIN_CCONLY_ADJUST_HORIZONTAL_ZOOM		998
#define ID_MAIN_CCONLY_ADJUST_VERTICAL_ZOOM			994
#define ID_MAIN_CCONLY_ADJUST_HORIZONTAL_SCROLL		997
#define ID_MAIN_CCONLY_ADJUST_VERTICAL_SCROLL		995
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM01			950
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM02			951
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM03			952
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM04			953
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM05			954
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM06			955
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM07			956
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM08			957
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM09			958
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM10			959
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM11			960
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM12			961
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM13			962
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM14			963
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM15			964
#define ID_MAIN_CCONLY_SELTRACK_FXPARAM16			965
#define ID_MAIN_CCONLY_SCRUB_JOG					992

#define ID_MAIN_GRID_ADJUST_2						40786
#define ID_MAIN_GRID_ADJUST_3						40787
#define ID_MAIN_GRID_ADJUST_1_2						40783
#define ID_MAIN_GRID_ADJUST_1_3						40784
#define ID_MAIN_GRID_SET_1							40779
#define ID_MAIN_GRID_SET_2							40780
#define ID_MAIN_GRID_SET_4							40781
#define ID_MAIN_GRID_SET_1_2						40778
#define ID_MAIN_GRID_SET_1_3						40777
#define ID_MAIN_GRID_SET_1_4						40776
#define ID_MAIN_GRID_SET_1_8						40775
#define ID_MAIN_GRID_SET_1_16						40774
#define ID_MAIN_GRID_SET_1_32						41047
#define ID_MAIN_GRID_SET_1_64						41048
#define ID_MAIN_OPTION_TOGGLE_GROUPING_OVERRIDE		1156
#define ID_MAIN_OPTION_TOGGLE_GRID_LINES			40145
#define ID_MAIN_OPTION_TOGGLE_METRONOME				40364
#define ID_MAIN_OPTION_TOGGLE_SNAP					1157
#define ID_MAIN_VIEW_TOGGLE_MIXER_VISIBLE			40078
#define ID_MAIN_VIEW_SHOW_TRACK_GROUPING_MATRIX		40768
#define ID_MAIN_VIEW_SHOW_PERFORMANCE_METER			40240
#define ID_MAIN_VIEW_SHOW_ROUTING_MATRIX			40251
#define ID_MAIN_VIEW_SHOW_NAVIGATOR_WINDOW			40268
#define ID_MAIN_VIEW_SHOW_UNDO_HISTORY				40072
#define ID_MAIN_CCONLY_METRONOME_VOLUME				999

#define ID_MAIN_TRACK_TOGGLE_FREEPOS				40641
#define ID_MAIN_ITEM_MOVE_ONETRACK_DOWN				40118
#define ID_MAIN_ITEM_MOVE_ONETRACK_UP				40117
#define ID_MAIN_ITEM_MOVE_GRIDSIZE_RIGHT			40794
#define ID_MAIN_ITEM_MOVE_GRIDSIZE_LEFT				40793
#define ID_MAIN_ITEM_GROUP							40032
#define ID_MAIN_ITEM_SELECTALL_INGROUP				40034
#define ID_MAIN_ITEM_REMOVE_FROMGROUP				40033
#define ID_MAIN_ITEM_SELECTALL_INTIMESEL			40717
#define ID_MAIN_ITEM_SELECTALL_INSELTRACK			40421
#define ID_MAIN_ITEM_UNSELECTALL					40289
#define ID_MAIN_ITEM_PITCH_ONECENT_DOWN				40207
#define ID_MAIN_ITEM_PITCH_OCTAVE_DOWN				40516
#define ID_MAIN_ITEM_PITCH_SEMITONE_DOWN			40205
#define ID_MAIN_ITEM_PITCH_ONECENT_UP				40206
#define ID_MAIN_ITEM_PITCH_OCTAVE_UP				40515
#define ID_MAIN_ITEM_PITCH_SEMITONE_UP				40204
#define ID_MAIN_ITEM_SELECT_NEXTRACK				40419
#define ID_MAIN_ITEM_SELECT_PREVTRACK				40418
#define ID_MAIN_ITEM_SELECT_NEXTITEM				40417
#define ID_MAIN_ITEM_SELECT_PREVITEM				40416
#define ID_MAIN_TAKE_ACTIVE_CROP					40131
#define ID_MAIN_TAKE_ACTIVE_DELETE					40129
#define ID_MAIN_TAKE_ACTIVE_NEXT					40125
#define ID_MAIN_TAKE_ACTIVE_PREVIOUS				40126
#define ID_MAIN_TAKE_REVERSE						41051

#define ID_MAIN_ITEM_GLUED_SELECTED						40362

#define ID_MAIN_ITEM_TOGGLE_MUTE						40175
#define ID_MAIN_ITEM_LOOP_SOURCE						40636
#define ID_MAIN_ITEM_LOOP_SECTION						40547
#define ID_MAIN_ITEM_RESET_PITCH						40653
#define ID_MAIN_ITEM_PROPERTIES							40009
#define ID_MAIN_TRACK_VIEW_FX							40291 
#define ID_MAIN_TRACK_VIEW_ENVELOPES					40292
#define ID_MAIN_TRACK_VIEW_IO							40293
#define ID_MAIN_TRACK_VIEW_GROUPING						40772
#define ID_MAIN_ENVELOPE_SHOW_ALL_ACTIVE				40888
#define ID_MAIN_ENVELOPE_TOGGLE_BYPASS					40883
#define ID_MAIN_ENVELOPE_TOGGLE_RECARM					40863
#define ID_MAIN_ENVELOPE_INSERT_POINT					40915
#define ID_MAIN_ENVELOPE_DELETE_POINTS					40333
#define ID_MAIN_ENVELOPE_CLEAR							40065
#define ID_MAIN_ENVELOPE_TOGGLE_DISPLAY					40884
#define ID_MAIN_ENVELOPE_TOGGLE_SHOW_ALL_ACTIVE			40890
#define ID_MAIN_ENVELOPE_TOGGLE_ALLVISIBLE_LANE			40891
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_LINEAR		40187
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_SQUARE		40188
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_SLOW			40425
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_FASTSTART	40430
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_FASTEND		40431
#define ID_MAIN_ENVELOPE_SET_DEFAULT_SHAPE_BEZIER		40681
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_LINEAR		40189
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_SQUARE		40190
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_SLOW		40424 
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_FASTEND		40429
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_FASTSTART	40428
#define ID_MAIN_ENVELOPE_SET_SELECTED_SHAPE_BEZIER		40683
#define ID_MAIN_ENVELOPE_UNSELECT_POINTS				40331
#define ID_MAIN_AUTOMATION_ACTIVE_FADER_CCONLY			986
#define ID_MAIN_AUTOMATION_ACTIVE_FADER_DEC				40858
#define ID_MAIN_AUTOMATION_ACTIVE_FADER_INC				40857
//#define ID_MAIN_AUTOMATION_MODE_TRIMREAD				40400
//#define ID_MAIN_AUTOMATION_MODE_READ					40401
//#define ID_MAIN_AUTOMATION_MODE_TOUCH					40402
//#define ID_MAIN_AUTOMATION_MODE_WRITE					40403
//#define ID_MAIN_AUTOMATION_MODE_LATCH					40404
#define ID_MAIN_ENVELOPE_TOGGLE_VISIBLE_VOLUME			40406
#define ID_MAIN_ENVELOPE_TOGGLE_VISIBLE_PAN				40407
#define ID_MAIN_ENVELOPE_TOGGLE_VISIBLE_PREFXVOL		40408
#define ID_MAIN_ENVELOPE_TOGGLE_VISIBLE_PREFXPAN		40409

#define ID_MAIN_SOLO_INFRONT_ADJUST_CCONLY				987 
#define ID_MAIN_SOLO_INFRONT							40745

#define ID_MAIN_MOVE_CURSOR_GRID_LEFT					40646
#define ID_MAIN_MOVE_CURSOR_GRID_RIGHT					40647

#define ID_MAIN_SHOW_ACTIONLIST							40605

#define ID_MAIN_INSERT_EDIT_MARKER			40171
#define ID_MAIN_INSERT_NEWTRACK				40001
#define ID_MAIN_INSERT_NEWMIDI				40214
#define ID_MAIN_MARKER_DELETE				40613
#define ID_MAIN_MARKERRGN_DELETE			40615
#define ID_MAIN_MARKERTIME_DELETE			40617
#define ID_MAIN_VZOOM_IN					40111
#define ID_MAIN_VZOOM_OUT					40112
#define ID_MAIN_GOTOTRACK_NEXT				40285
#define ID_MAIN_GOTOTRACK_PREV				40286
#define ID_MAIN_SCROLL_UP					40138
#define ID_MAIN_SCROLL_DOWN					40139
#define ID_MAIN_SCROLL_LEFT					40140
#define ID_MAIN_SCROLL_RIGHT				40141
#define ID_MAIN_MIDI_INSERT_ITEM			40214
#define ID_MAIN_MIDI_OPEN_ITEMS				40904
#define ID_MAIN_MIDI_NOTE_MOVELEFT			40183
#define ID_MAIN_MIDI_NOTE_MOVERIGHT			40184
#define ID_MAIN_MIDI_NOTE_SELECTNEXT		40413
#define ID_MAIN_MIDI_NOTE_SELECTPREV		40414
#define ID_MAIN_MIDI_NOTE_VALUE_SELNEXT		40428
#define ID_MAIN_MIDI_NOTE_VALUE_SELPREV		40427

#define ID_MIDI_SETCCLANE_VELOCITY			40236
#define ID_MIDI_SETCCLANE_000_BANKSELECT	40237
#define ID_MIDI_SETCCLANE_001_MODWHEEL		40238
#define ID_MIDI_SETCCLANE_119				40356
#define ID_MIDI_SETCCLANE_PITCH				40365
#define ID_MIDI_SETCCLANE_PROGRAM			40366
#define ID_MIDI_SETCCLANE_CHANNELPRES		40367
#define ID_MIDI_SETCCLANE_BANKPROGSELECT	40368
#define ID_MIDI_SETCCLANE_TEXTEVENTS		40369
#define ID_MIDI_SETCCLANE_SYSEX				40370

#define ID_SWS_MAIN_SHOW_TRACKLIST						"SWSTL_OPEN"
#define ID_SWS_MAIN_SHOW_MARKERLIST						"SWSMARKERLIST1"
#define ID_SWS_MAIN_SHOW_SNAPSHOTLIST					"SWSSNAPSHOT_OPEN"
#define ID_SWS_MAIN_NEW_SNAPSHOT_NAMED					"SWSSNAPSHOT_NEWEDIT"
#define ID_SWS_MAIN_SAVE_SNAPSHOT						"SWSSNAPSHOT_SAVE"
#define ID_SWS_MAIN_TOGGLE_ZOOM_SELITEM					"SWS_TOGZOOMI"
#define ID_SWS_MAIN_TOGGLE_ZOOM_SELTRACK_TIME			"SWS_TOGZOOMTT"

#define AUTOMAPSLDISPLAY_WIDTH	72

#define BUFFER_SIZE							1000
#define MUTEX_TIMEOUT						INFINITE

class AutomapCSurf;
class AutomapDevice;

static HANDLE gMutex;

void MidiEdit_OnCommand(int command, int flags);											//!< \brief Send commands to the MIDI window
void KBD_OnMidiEditActionEx(int cmd, int val, int valhw, int relmode, ReaProject* proj);	//!< \brief Send extended commands to the MIDI window \bug Doesn't work
void Main_OnExtendedCommand(const char* actionName, int flags);								//!< \brief Send extended commands to the main window
double Midi2Val(int midi);																	//!< \brief Convert MIDI value 0-127 to normalized value 0.0-1.0
int Val2Midi(double value);																	//!< \brief Convert normalized value 0.0-1.0 to MIDI value 0-127

//! \brief Automap client
//! \todo Secure "_connection = NULL" scenario ?
//! \bug Novation: Automap Server crashes when active page gets deleted by remapping
//! \todo MIDI page: 'Prev/Next CC Lane', 'join MIDI notes', 'reverse', 'rect/diamonds/triangles'
//! \todo Use controller->Get/SetValue() more often (vs CSurf attribute)
//! \todo Tempo change
//! \bug Reaper: MIDI Editor action says 'Grid: set division to 7' but sets to 6
class AutomapReaperClient : public Automap::Client
{
	friend class AutomapCSurf;

	private:
		AutomapCSurf*				_parent;			//!< Parent surface
		Automap::Connection*		_connection;		//!< Automap connection
		AutomapDevice*				_device;			//!< Automap device 

		AutomapReaperClient();							//!< Constructor (forbidden access)

	public:
		int _nbParams;									//!< Number of parameters
		string _instanceName;							//!< Instance name

	public:
		AutomapReaperClient(AutomapCSurf* parent);		//!< Constructor
		~AutomapReaperClient();							//!< Destructor

		static void strcpyCrop(char* buffer, const char* source);			//!< Safe strcpy: will crop string to AUTOMAP_NAME_LENGTH chars
		static void sprintfCrop(char* buffer, const char* message, ...);	//!< Safe sprintf: will crop string to AUTOMAP_NAME_LENGTH chars

		AutomapDevice* getDevice();											//!< Return Automap device 

		void getTrackNameCrop(MediaTrack* trackptr, char buffer[AUTOMAP_NAME_LENGTH+1], int length = AUTOMAP_NAME_LENGTH, int *TrackId = NULL);		//!< Get track name cropped to \<length\> chars
		bool getTrackSendParamName(MediaTrack *tr, ReaperControlType controlType, int slot, char* paramName);		//!< Get track send parameter name

		double getTrackSendValue(MediaTrack *tr, ReaperControlType controlType, int slot, char* valueTextOut);		//!< Get Track (Hardware) Send value
		double getFxParamValue(MediaTrack *tr, ReaperControlType controlType, int slot, char* valueTextOut);		//!< Get FX Parameter value
		void setTrackSendValue(MediaTrack *tr, ReaperControlType controlType, int slot, double value);				//!< Set Track (Hardware) Send value
		void setFxParamValue(MediaTrack *tr, ReaperControlType controlType, int slot, double value);				//!< Set FX Parameter value

		double vol2Value(double volume, char* valueTextOut = NULL);			//!< Convert vol (dB) to normalized value 
		double value2Vol(double value);										//!< Convert normalized value to vol (dB)
		double pan2Value(double pan, char* valueTextOut = NULL);			//!< Convert pan (%) to normalized value 
		double value2Pan(double value);										//!< Convert normalized value to pan (%)
		double boolStatus2Value(bool status, char* valueTextOut = NULL);	//!< Convert bool to normalized value
		double sendMode2PotVal(int mode);									//!< Convert Send Mode int to normalized value
		int potVal2SendMode(double value);									//!< Convert Normalized value to Send Mode int

		void setFfwRew(int direction, int mode, bool active);				//!< Set FFW/REW flags/actions

		// Automap::Client Compliance:

		void GetName(char *name) const;					//!< Get client name
		void GetMfr(char *mfr) const;					//!< Get client manufacturer
		void GetType(char *type) const;					//!< Get client type (Mixer, User, Fx, Inst...)
		void GetSaveName(char *name) const;				//!< Get client save name
		Automap::GUID GetGUID() const;					//!< Get client GUID
		
		void GetInstanceName(char *name) const;			//!< Get instance name
		void SaveInstanceName(const char *name);		//!< Save instance name

		int GetNumParams() const;						//!< Get number of parameters
		Automap::Connection* GetConnection();			//!< Get client connection pointer

		bool GetParamInfo(int idx, Automap::ParamInfo& data);						//!< Get parameter info
		float GetParamValue(int idx, char valueTextOut[AUTOMAP_NAME_LENGTH+1]);		//!< Get parameter value (and display) \note Registered for Update() after call to SendEvent(idx)
		void SetParamValue(int idx, float value);									//!< Set parameter value

		int GetNumPresets() const;													//!< Get number of presets
		int GetCurrentPreset(char name[AUTOMAP_NAME_LENGTH+1]) const;				//!< Get current preset
		void SetCurrentPreset(int n);												//!< Set current preset

		bool SupportsTransport(AutomapTransport type) const;						//!< Transport config function
		void OnTransport(AutomapTransport type, float param);						//!< Handler for transport events
};

//! \brief Automap Raw MIDI Client. Works only for SL series.

//! F0 00 20 29 03 03 20 02 (PID) 00 01 01 F7 - online\n
//! F0 00 20 29 03 03 20 02 (PID) 00 01 00 F7 - offline\n 
//! F0 00 20 29 03 03 20 02 (PID) 00 02 02 (04/05) F7 - clear (left/right)\n 
//! F0 00 20 29 03 03 20 02 (PID) 00 02 01 (pos) (rowID) 04 (t e x t) F7 - write text\n 
//! F0 7E 7F 06 01 F7 - Universal Device Inquiry (ReMOTE SL should respond with a 17-byte message {00 20 29} {03 00} {03 00} )\n\n
//! PIDs:\n
//! - 01: Reason\n
//! - 02: Automap Server\n
//! - 03: Logic / Mixer Automap\n
//! - 04: Ableton\n
class AutomapSLRawMidiClient
{
	private:
		//! \brief Extended MIDI_event_t for sysex
		struct MidiEvent
		{
			MIDI_event_t evt;			//!< Original MIDI_event_t struct
			char data[BUFFER_SIZE];		//!< Extra memory alloc

			MidiEvent();				//!< Constructor
		};

		//! \brief 17-byte MIDI buffer for Universal Device Inquiries
		struct UniversalDeviceInquiryReply
		{
			char hdr[5];				//!< Universal Device Reply (F0 7E 7F 06 02?)
			char man[3];				//!< Manufacturer ID
			char family[2];				//!< Family ID, LSB first
			char member[2];				//!< Family Member, LSB first
			char rev[4];				//!< Software revision, ASCI (ex. 30 31 30 30  = '0100' = 1.00)
			char eox;					//!< End-Of-Exclusive (F7)

			bool isRemoteSL();
		}; 

	public:
		typedef enum Panel { ePANEL_LEFT = 0x04, ePANEL_RIGHT = 0x05 };														//!< Panel identifier (for clear)
		typedef enum Row { ePANEL_TOPLEFT = 1, ePANEL_BOTTOMLEFT = 2, ePANEL_TOPRIGHT = 3, ePANEL_BOTTOMRIGHT = 4 };		//!< Row/Panel identifier (for write)

	private:
		midi_Output *_midiout;				//!< MIDI Output handler

	private:
		AutomapSLRawMidiClient();			//!< Constructor (forbidden access)

		static void appendMidiMsg(MidiEvent &msg, int nbBytes, ...);		//!< Add data to MIDI message
		static bool isValidInPort(int portId);								//!< Return true if PortId is a valid MIDI Input port
		static bool isValidOutPort(int portId);								//!< Return true if PortId is a valid MIDI Output port
		static void sendUniversalDeviceInquiry(midi_Output* midiout);		//!< Send Universal Device Inquiry message

	public:
		AutomapSLRawMidiClient(int midiPort, int *errStats);					//!< Constructor 
		~AutomapSLRawMidiClient();												//!< Destructor 

		static bool detectDeviceMidiPorts(int &midiPortIn, int &midiPortOut);	//!< Autodetect device by sending Universal Device Inquiry messages
		void clearPanel(Panel panel);											//!< Clear panel
		void sendText(Row row, string text, unsigned char pos = 0x00);			//!< Write text on device
};

//! \brief IReaperControlSurface class for Automap
//! \todo Use Automap presets/GUID?
//! \todo Use MIDI Sysex to detect device
//! \bug Reaper: purpose of CSurf_OnTrackSelection() / IReaperControlSurface::OnTrackSelection() compared
//! to CSurf_SetSurfaceSelected() / CSurf_OnSelectedChange() / IReaperControlSurface::SetSurfaceSelected() ?
//! \bug Reaper: IReaperControlSurface::OnTrackSelection() reacts to UI select and CSurf_OnTrackSelection() but not actions/shortcuts
//! \bug Reaper: action 'Automation lane: set active fader (midi CC only)' is not working as it should in absolute mode: 0:-50%, 1:+50%, 2:+150%, ... 127:+12650%
//! \todo FR: Novation: New AutomapParamInfo:: attributes for button Normal/Momentary/Toggle/Step config and on/off values
//! \todo FR: Novation: New Client:: method to send text to LCD screen (currently only available through sysex)
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on "save as..."
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on FX chain on/off
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on FX selection
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Send Vol change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Send Pan change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Send Mute change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Send Pan change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Hardware Send Vol change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Hardware Send Pan change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Hardware Send Mute change
//! \todo FR: Cockos: IReaperControlSurface::SetSurfaceXXX() notification on Track Hardware Send Pan change
//! \todo FR: Cockos: New TCP/MCP FX parameter functions + CSurf notifications
//! \todo FR: Cockos: Envelope: move selected points up/down
//! \todo FR: Cockos: Envelope: select prev/next point
//! \bug BR: Novation: Cannot setup momentary button with a step size of zero (?)
//! \bug BR: Novation: Automap Server crashes when active page is deleted by remapping (?)
//! \bug BR: Cockos: GetSetTrackSendInfo("I_SENDMODE") does not refresh IO window
//! \bug BR: Cockos: CSurf_NumTracks() always work in TCP mode until mixer window has been in focus
class AutomapCSurf : public IReaperControlSurface
{
	public:
		//! \brief Automap CSurf settings
		struct Parameters
		{
			//! \brief FFW/RWD mode
			typedef enum FfwRwdMode {eFFWRWD_STANDARD = 0, eFFWRWD_MOVECURSOR, eFFWRWD_STARTEND, eFFWRWD_NEXTPREVMARKER, eFFWRWD_NBMODES };		

			AutomapDevice::Type deviceType;				//!< Device type (RemoteSL, iPhone, etc)
			bool hasGlobalPages;						//!< Add global pages to mapping if true
			bool mcpMode;								//!< Track/Mixer view mode flag
			double zoomFactor;							//!< Zoom factor
			FfwRwdMode ffwRwdMode;						//!< FFW/RWD mode
			double ffwRwdRange;							//!< FFW/RWD range (for 'move cursor' mode)
			int seekplay;								//!< FFW/RWD seekplay (for 'standard' mode, 0=slow/1=normal)
			int midiOut;								//!< Automap MIDI output port (used for sysex display messages: SL only)
			int fnMidiChannel;							//!< MIDI Channel for Fn/FnPot/FnEnc assignable controls
			map<int,string> fnButAliases;				//!< Fn buttons aliases
			map<int,string> fnPotAliases;				//!< Fn pots aliases
			map<int,string> fnEncAliases;				//!< Fn encoders aliases
int speedDialFactor;							//!< Speed dial factor

			Parameters();								//!< Constructor
		};

		static Parameters _parameters;								//!< Settings
		static AutomapSLRawMidiClient* _automapRawMidiClient;		//!< Raw MIDI Client (used for text send on SL series)

		static void loadConfigFile();				//!< Load config file (Fn MIDI channel/aliases)
		static void saveConfigFile();				//!< Save config file (Fn MIDI channel/aliases)

	private :
		static bool _doFfw;								//!< FFW status for main loop
		static bool _doRew;								//!< REW status for main loop

		AutomapReaperClient* _automapClient;			//!< Automap client pointer
		char _configString[512];						//!< Config string
		set<AutomapController*> _updateCtrls;			//!< List of controllers to be updated in main loop (e.g. endless encoders)
		map<int, pair<double,double> > _fxParamsMinMax;	//!< Min/Max values of current FX parameters
		int _nbTracksMax;								//!< Max number of tracks since creation \note Used to handle Automap Server crash when removing active page from mapping

		int _selTrack;									//!< Current Track selection
		int _selFx;										//!< Current FX selection
		int _selFxParam;								//!< Current FX param selection
		int _selTcpFxParam;								//!< Current TCP/MCP FX param selection
		int _selTrackSend;								//!< Current Track Send selection
		bool _selTrackSendMute;							//!< Current Track Send selection mute status
		int _selTrackSendMode;							//!< Current Track Send selection mode (pre-fx, post-fader, etc.)
		int _selTrackHardSend;							//!< Current Track Hardware Send selection
		bool _selTrackHardSendMute;						//!< Current Track Hardware Send selection mute status
		int _selTrackHardSendMode;						//!< Current Track Hardware Send selection mode (pre-fx, post-fader, etc.)
		int _midiViewMode;								//!< Midi View Mode
		int _midiGrid;									//!< Midi Grid
int _mainGrid;									//!< Main Grid
int _defaultEnvShape;
int _selectedEnvShape;
	private:
		static void setDoubleThreadSafe(double *dest, double val);		//!< Thread-safe member write (double)
		static double getDoubleThreadSafe(double *value);				//!< Thread-safe member read (double)
		static void setIntThreadSafe(int *dest, int val);				//!< Thread-safe member write (int)
		static int getIntThreadSafe(int *value);						//!< Thread-safe member read (int)
		static void setBoolThreadSafe(bool *dest, bool val);			//!< Thread-safe member write (bool)
		static bool getBoolThreadSafe(bool *value);						//!< Thread-safe member read (bool)

		void clearUpdateCtrls();										//!< Clear controllers refresh queue
		void updateControllers();										//!< Update controllers in queue (thread-safe)
		void clearSelectedTracks();										//!< Clear selected tracks

	public:
		AutomapCSurf(int *errStats);					//!< Constructor
		~AutomapCSurf();								//!< Destructor

		static int parametersToConfigFlags(AutomapCSurf::Parameters &parameters);						//!< Convert CSurf setting booleans to to config flag (32 max)
		static void configFlagsToParameters(int configFlags, AutomapCSurf::Parameters &parameters);		//!< Convert config flag to CSurf setting booleans (32 max)
		static void parametersToConfigString(Parameters &parameters, char* configString);				//!< Convert CSurf settings to config string
		static bool configStringToParameters(const char* configString, Parameters &parameters);			//!< Convert config string to CSurf settings 

		// Thread-safe parameters access:

		static void setDoFfw(bool status);				//!< Set FFW status flag (used in main loop)
		static bool getDoFfw();							//!< Get FFW status flag (used in main loop)
		static void toggleDoFfw();						//!< Invert FFW status flag (used in main loop)
		static void setDoRew(bool status);				//!< Set REW status flag (used in main loop)
		static bool getDoRew();							//!< Get REW status flag (used in main loop)
		static void toggleDoRew();						//!< Invert REW status flag (used in main loop)
		static void setMcpMode(bool val);				//!< Set Track/Mixer view mode
		static bool getMcpMode();						//!< Get Track/Mixer view mode
		static void toggleMcpMode();					//!< Invert Track/Mixer view mode
		static void setFfwRwdMode(int mode);			//!< Set FFW/REW mode
		static int getFfwRwdMode();						//!< Get FFW/REW mode
		static void toggleFfwRwdMode();					//!< Invert FFW/REW mode
		static void setZoomFactor(double factor);		//!< Set Zoom Factor
		static double getZoomFactor();			 		//!< Get Zoom Factor
		static void increaseZoomFactor();		 		//!< Increase Zoom Factor (2x)
		static void decreaseZoomFactor(); 				//!< Decrease Zoom Factor (2x)
static void setSpeedDialFactor(int factor);			//!< Set Speed Dial factor
static int getSpeedDialFactor();			 		//!< Get Speed Dial factor
static void increaseSpeedDialFactor();		 		//!< Increase Speed Dial factor (2x)
static void decreaseSpeedDialFactor(); 				//!< Decrease Speed Dial factor (2x)
		static void setFfwRwdRange(double factor);		//!< Set FFW/REW Range
		static double getFfwRwdRange();			  		//!< Get FFW/REW Range
		static void increaseFfwRwdRange();		  		//!< Increase FFW/REW Range (2x)
		static void decreaseFfwRwdRange();				//!< Decrease FFW/REW Range (2x)
		static int getFnMidiChannel();		  			//!< Get Fn/FnPot/FnEnc assignable controls MIDI Channel
		static void setFnMidiChannel(int channel);		//!< Set Fn/FnPot/FnEnc assignable controls MIDI Channel
		static void setFnButAlias(int slot, const char* alias);		//!< Set Fn button alias
		static const char* getFnButAlias(int slot);					//!< Get Fn button alias
		static void setFnPotAlias(int slot, const char* alias);		//!< Set Fn pot alias
		static const char* getFnPotAlias(int slot);					//!< Get Fn pot alias
		static void setFnEncAlias(int slot, const char* alias);		//!< Set Fn encoder alias
		static const char* getFnEncAlias(int slot);			   		//!< Get Fn encoder alias
		static void resetFnAliases();								//!< Reset Fn aliases

		// Thread-safe member access:

		void setSelTrack(int val);						//!< Set current Track selection
		int getSelTrack();		  						//!< Get current Track selection
		void setSelFx(int val);							//!< Set current FX selection
		int getSelFx();									//!< Get current FX selection
		void setSelFxParam(int val);					//!< Set current FX param selection
		int getSelFxParam();							//!< Get current FX param selection
		void setSelTcpFxParam(int val);					//!< Set current TCP/MCP FX param selection
		int getSelTcpFxParam();							//!< Get current TCP/MCP FX param selection
		bool hasFxParamMinMax(int fxParam);				//!< Return true if current FX param selection has min/max
		void getFxParamMinMax(int fxParam, double *minval, double *maxval);		//!< Get current FX param selection min/max
		void setFxParamMinMax(int fxParam, double minval, double maxval);		//!< Set current FX param selection min/max
		void setSelTrackSend(int val);					//!< Set current Track Send selection
		int getSelTrackSend();		  					//!< Get current Track Send selection
		void setSelTrackSendMode(int val);				//!< Set current Track Send selection mute status
		int getSelTrackSendMode();		  				//!< Get current Track Send selection mute status
		void setSelTrackSendMute(bool val);				//!< Set current Track Send selection mode (pre-fx, post-fader, etc.)
		bool getSelTrackSendMute();		   				//!< Get current Track Send selection mode (pre-fx, post-fader, etc.)
		void setSelTrackHardSend(int val);				//!< Set current Track Hardware Send selection
		int getSelTrackHardSend();		  				//!< Get current Track Hardware Send selection
		void setSelTrackHardSendMode(int val);			//!< Set current Track Hardware Send selection mute status
		int getSelTrackHardSendMode();	  				//!< Get current Track Hardware Send selection mute status
		void setSelTrackHardSendMute(bool val);			//!< Set current Track Hardware Send selection mode (pre-fx, post-fader, etc.)
		bool getSelTrackHardSendMute();	  				//!< Get current Track Hardware Send selection mode (pre-fx, post-fader, etc.)
		void setMidiViewMode(int view);					//!< Set Midi View Mode
		int getMidiViewMode();	  						//!< Get Midi View Mode
void setDefaultEnvShape(int shape);
int getDefaultEnvShape();
void setSelectedEnvShape(int shape);
int getSelectedEnvShape();

		void setMidiGrid(int grid);						//!< Set Midi Grid
		int getMidiGrid();	  							//!< Get Midi Grid
void setMainGrid(int grid);						//!< Set Main Grid
int getMainGrid();	  							//!< Get Main Grid

void resetMapping(int preset = -1);

		void selectTrack(int trackIndex);									//!< Select track (and deselect the others)
		void selectFx(MediaTrack* trackid, int fx);							//!< Select FX of current track
		void selectFxParam(MediaTrack* trackid, int param);					//!< Select FX Param of current track (used as an offset if 1+ FX Param slots are available)
		void selectTcpFxParam(int param);									//!< Select TCP/MCP FX Param of current track (used as an offset if 1+ UI FX Param slots are available)
		void selectTrackSend(MediaTrack* trackid, int sendId);				//!< Select Track Send of current track (used as an offset if 1+ Track Send slots are available)
		void selectHardTrackSend(MediaTrack* trackid, int hardSendId);		//!< Select Track Hardware Send of current track (used as an offset if 1+ Track Hardware Send slots are available)
		void addUpdateCtrl(AutomapController* controller);					//!< Add controller to refresh queue for main loop
		void selectMidiGrid(int grid);										//!< Select Midi Grid
void selectMainGrid(int grid);										//!< Select Main Grid
		void selectMidiCcLane(int lane);									//!< Select MIDI CC Lane

		// IReaperControlSurface compliance:

		const char *GetTypeString();							//!< Get type string
		const char *GetDescString();							//!< Get description string
		const char *GetConfigString();							//!< Get string of configuration data

		void CloseNoReset();				//<! Close without sending "reset" messages, prevent "reset" being sent on destructor
		void Run();							//<! called 30x/sec or so.

		void SetTrackListChange();										//!< Called by the host when track list changes
		void SetSurfaceVolume(MediaTrack *trackid, double volume);		//!< Called by the host when volume changes
		void SetSurfacePan(MediaTrack *trackid, double pan);			//!< Called by the host when pan changes
		void SetSurfaceMute(MediaTrack *trackid, bool mute);			//!< Called by the host when mute changes
		void SetSurfaceSelected(MediaTrack *trackid, bool selected);	//!< Called by the host when track selection changes
		void SetSurfaceSolo(MediaTrack *trackid, bool solo);			//!< Called by the host when solo changes
		void SetSurfaceRecArm(MediaTrack *trackid, bool recarm);		//!< Called by the host when Rec Arm changes
		void SetPlayState(bool play, bool pause, bool rec);				//!< Called by the host when Play/Pause/Rec changes
		void SetRepeatState(bool rep);									//!< Called by the host when Repeat (Loop) changes
		void SetTrackTitle(MediaTrack* trackid, const char *title);		//!< Called by the host when track title changes
		void SetAutoMode(int mode);										//!< Called by the host when automation mode for current track changes
		bool GetTouchState(MediaTrack *trackid, int isPan);				//!< Called by the host when touch state changes

		void ResetCachedVolPanStates();									//!< Good to flush your control states here
		void OnTrackSelection(MediaTrack *trackid);						//!< Track was selected \note NOT called by Main_OnCommand() actions (only by mouse select and CSurf_OnTrackSelection())
		bool IsKeyDown(int key);										//!< To send key msgs to host from surface (values: VK_CONTROL, VK_MENU, VK_SHIFT, etc)

		int Extended(int call, void *parm1, void *parm2, void *parm3);	//!< Use for extended calls (return 0 if unsupported)
};

#endif	// __AUTOMAP_CSURF_H__
