/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#ifndef __AUTOMAP_DEVICE_H__
#define __AUTOMAP_DEVICE_H__

#include <set>
#include <map>
using namespace std;

#include "AutomapSDK/include/Automap.h"

#define AUTOMAP_MAXNBPARAM						2048
#define AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE		48
#define AUTOMAP_REMOTESL_NBTRACKS_PER_PAGE		8
#define AUTOMAP_REMOTESL_NBGLOBALPAGES			1
#define AUTOMAP_IPHONE_NBPARAMS_PER_PAGE		10
#define AUTOMAP_IPHONE_NBTRACKS_PER_PAGE		1
#define AUTOMAP_IPHONE_NBGLOBALPAGES			2
#define AUTOMAP_NOCTURN_NBPARAMS_PER_PAGE		16
#define AUTOMAP_NOCTURN_NBTRACKS_PER_PAGE		4
#define AUTOMAP_NOCTURN_NBGLOBALPAGES			2

class AutomapReaperClient;
class AutomapCSurf;

//! \brief Set of Reaper controls used (or not) by the CSurf
//! \note Do NOT change the order of eCTRL_TRACKSEND_* and eCTRL_TRACKHARDSEND_* entries! NEVER!!!
typedef enum {
				// Track controllers
				eCTRL_VOL, eCTRL_PAN, eCTRL_SOLO, eCTRL_MUTE, eCTRL_ARM,
eCTRL_PHASE,

				// Track selection controllers
				eCTRL_CURSEL_VOL, eCTRL_CURSEL_PAN, eCTRL_CURSEL_SOLO, eCTRL_CURSEL_MUTE, eCTRL_CURSEL_ARM,
				eCTRL_CURSEL_FXBYPASS,
eCTRL_CURSEL_PHASE,
				eCTRL_CURSEL_FXSELECT_WHEEL, eCTRL_CURSEL_FXPARAMSELECT_WHEEL,
				eCTRL_CURSEL_TCPFXPARAMSELECT_WHEEL,
				eCTRL_CURSEL_TRACKSENDSELECT_WHEEL, eCTRL_CURSEL_TRACKHARDSENDSELECT_WHEEL,
				eCTRL_CURSEL_NAME,
eCTRL_CURSEL_VIEW_ENVELOPES, eCTRL_CURSEL_VIEW_FX, eCTRL_CURSEL_VIEW_IO,
eCTRL_CURSEL_VIEW_GROUPING,
eCTRL_CURSEL_AUTOMATION_MODE,

				//Slot-based controllers (watch out for multiple assigns: number of slots is constant!)
				eCTRL_ASSIGNABLEFN,			// is multiple assign-safe
				eCTRL_ASSIGNABLEFNPOT,		// is multiple assign-safe
				eCTRL_ASSIGNABLEFNENC,		// is multiple assign-safe
				eCTRL_FXPARAM,
				eCTRL_TCPFXPARAM,
				eCTRL_TCPFXPARAM_ENC,
				// Don't change the order !!!!
				eCTRL_TRACKSEND_VOL, eCTRL_TRACKSEND_PAN, eCTRL_TRACKSEND_MUTE, eCTRL_TRACKSEND_MODE,
				eCTRL_TRACKSEND_MUTE_POT, eCTRL_TRACKSEND_MODE_POT,
				eCTRL_TRACKHARDSEND_VOL, eCTRL_TRACKHARDSEND_PAN, eCTRL_TRACKHARDSEND_MUTE, eCTRL_TRACKHARDSEND_MODE,
				eCTRL_TRACKHARDSEND_MUTE_POT, eCTRL_TRACKHARDSEND_MODE_POT,

				// Global controllers
				eCTRL_INSERT_MARKERRGN, eCTRL_INSERT_MARKER, eCTRL_LOOP_SETSTART, eCTRL_LOOP_SETEND,
				eCTRL_DELETE_MARKER,
				eCTRL_MARKER_PREV, eCTRL_MARKER_NEXT, eCTRL_PREVNEXTMARKER_WHEEL,
				eCTRL_HZOOM_WHEEL, eCTRL_HZOOM_POT, eCTRL_VZOOM_WHEEL, eCTRL_VZOOM_POT,
				eCTRL_VSCROLL_WHEEL, eCTRL_VSCROLL_POT, eCTRL_HSCROLL_WHEEL, eCTRL_HSCROLL_POT,
				eCTRL_FFW, eCTRL_RWD, eCTRL_FFWRWD_WHEEL,
				eCTRL_PLAY, eCTRL_STOP, eCTRL_RECORD, eCTRL_GOSTART, eCTRL_GOEND, eCTRL_LOOP,
				eCTRL_CLEARRECARM, eCTRL_MCPMODE,
				eCTRL_TRACKSELECT, eCTRL_TRACKSELECT_WHEEL,
				eCTRL_SUBMENUSELECT_WHEEL,
				eCTRL_SCRUB_JOG_WHEEL, eCTRL_SCRUB_JOG_POT,
eCTRL_GRID_WHEEL, eCTRL_GROUPING_OVERRIDE, eCTRL_GRID_LINES, eCTRL_METRONOME, eCTRL_SNAP,
eCTRL_MIXER_VISIBLE, eCTRL_TRACK_GROUPING_MATRIX, eCTRL_PERFORMANCE_METER, eCTRL_ROUTING_MATRIX, eCTRL_NAVIGATOR_WINDOW,
eCTRL_METRONOME_VOLUME,
eCTRL_SWS_TRACKLIST,

eCTRL_TRACK_FREEPOS, eCTRL_ITEM_MOVE_DOWN, eCTRL_ITEM_MOVE_UP, eCTRL_ITEM_MOVE_RIGHT, eCTRL_ITEM_MOVE_LEFT,
eCTRL_ITEM_GROUP, eCTRL_ITEM_SELECTALL_INGROUP, eCTRL_ITEM_REMOVE_FROMGROUP, eCTRL_ITEM_SELECTALL_INTIMESEL,
eCTRL_ITEM_SELECTALL_INSELTRACK, eCTRL_ITEM_UNSELECTALL,
eCTRL_ITEM_PITCH_ONECENT_DOWN, eCTRL_ITEM_PITCH_OCTAVE_DOWN, eCTRL_ITEM_PITCH_SEMITONE_DOWN,
eCTRL_ITEM_PITCH_ONECENT_UP, eCTRL_ITEM_PITCH_OCTAVE_UP, eCTRL_ITEM_PITCH_SEMITONE_UP,
eCTRL_ITEM_SELECT_NEXTRACK, eCTRL_ITEM_SELECT_PREVTRACK, eCTRL_ITEM_SELECT_NEXTITEM, eCTRL_ITEM_SELECT_PREVITEM,
eCTRL_TAKE_ACTIVE_CROP, eCTRL_TAKE_ACTIVE_DELETE, eCTRL_TAKE_ACTIVE_NEXT, eCTRL_TAKE_ACTIVE_PREVIOUS,
eCTRL_ITEM_PITCH_ONECENT_WHEEL, eCTRL_ITEM_PITCH_OCTAVE_WHEEL, eCTRL_ITEM_PITCH_SEMITONE_WHEEL,
eCTRL_ITEM_MUTE, eCTRL_ITEM_LOOP_SOURCE, eCTRL_ITEM_LOOP_SECTION,
eCTRL_ITEM_PROPERTIES,
eCTRL_TAKESELECT_WHEEL, eCTRL_ITEMSELECT_WHEEL,
eCTRL_TAKE_REVERSE,

eCTRL_SHOW_ALL_ACTIVE_ENVELOPES,
eCTRL_ENVELOPE_BYPASS, eCTRL_ENVELOPE_RECARM,
eCTRL_ENVELOPE_INSERT_POINT,

eCTRL_ENVELOPE_DEFAULT_SHAPE,
eCTRL_ENVELOPE_SELECTED_SHAPE,

eCTRL_ENVELOPE_VISIBLE_VOLUME, eCTRL_ENVELOPE_VISIBLE_PAN, eCTRL_ENVELOPE_VISIBLE_PREFXVOL, eCTRL_ENVELOPE_VISIBLE_PREFXPAN,
eCTRL_ENVELOPE_CLEAR, eCTRL_ENVELOPE_DELETE_POINTS, eCTRL_ENVELOPE_TOGGLE_DISPLAY, eCTRL_ENVELOPE_ALLVISIBLE_LANE,

eCTRL_AUTOMATION_ACTIVE_FADER_POT, eCTRL_AUTOMATION_ACTIVE_FADER_WHEEL,

eCTRL_UNDO_HISTORY,

eCTRL_SWS_MARKERLIST, eCTRL_SWS_SNAPSHOTLIST,

eCTRL_SWS_ZOOM_TO_TRACKTIMESELECT, eCTRL_SWS_NEW_SNAPSHOT, eCTRL_SWS_SAVE_SNAPSHOT,

eCTRL_SOLO_INFRONT_ADJUST_POT,

eCTRL_MOVE_CURSOR_GRID_WHEEL,

eCTRL_SHOW_ACTIONLIST,
eCTRL_ITEM_GLUED_SELECTED,

eCTRL_SPEED_DIAL,

//eCTRL_ITEM_RESET_PITCH,
//eCTRL_ENVELOPE_UNSELECT_POINTS,

				// MIDI Controllers
				eMIDICTRL_VZOOM_WHEEL, eMIDICTRL_HZOOM_WHEEL, eMIDICTRL_VSCROLL_WHEEL, eMIDICTRL_HSCROLL_WHEEL,
				eMIDICTRL_VMOVE_WHEEL, eMIDICTRL_HMOVE_WHEEL,
				eMIDICTRL_MOVE_UP, eMIDICTRL_MOVE_DOWN, eMIDICTRL_MOVE_RIGHT, eMIDICTRL_MOVE_LEFT,
				eMIDICTRL_GRID_WHEEL,
				eMIDICTRL_VELOCITY_FINE_WHEEL, eMIDICTRL_VELOCITY_COARSE_WHEEL,
				eMIDICTRL_VELOCITY_INC_FINE, eMIDICTRL_VELOCITY_INC_COARSE, eMIDICTRL_VELOCITY_DEC_FINE, eMIDICTRL_VELOCITY_DEC_COARSE,
				eMIDICTRL_TOGGLE_VIEWMODE, eMIDICTRL_TOGGLE_SNAP, eMIDICTRL_TOGGLE_GRID,
				eMIDICTRL_INSERTNOTE, eMIDICTRL_NOTEPROPERTIES, eMIDICTRL_NEXTSECONDARY,
				eMAINMIDICTRL_INSERTITEM, eMAINMIDICTRL_OPENITEMS,
				eMIDICTRL_TRANSPOSE_INC1, eMIDICTRL_TRANSPOSE_DEC1, eMIDICTRL_TRANSPOSE_INC12, eMIDICTRL_TRANSPOSE_DEC12,
				eMIDICTRL_HUMANIZE, eMIDICTRL_FIT_CONTENT, eMIDICTRL_FIT_LOOP, eMIDICTRL_DELETENOTE,
				eMIDICTRL_NOTE_MOVELEFT, eMIDICTRL_NOTE_MOVERIGHT, eMIDICTRL_NOTE_SELECTNEXT, eMIDICTRL_NOTE_SELECTPREV,
				eMIDICTRL_NOTE_VALUE_SELNEXT, eMIDICTRL_NOTE_VALUE_SELPREV,
				eMIDICTRL_NOTE_LENGTH_WHEEL,
				eMIDICTRL_CCLANE_POT,

				eCTRL_UNKNOWN } ReaperControlType;

typedef enum {	eGRID_4 = 0, eGRID_2, eGRID_1, eGRID_1_2, eGRID_1_3, eGRID_1_4, eGRID_1_4T, eGRID_1_8, eGRID_1_8T, eGRID_1_16, eGRID_1_16T,
				eGRID_1_32, eGRID_1_32T, eGRID_1_64, eGRID_1_64T, eGRID_1_128, eGRID_1_128T, eGRID_LAST } GridDivision;

//! \brief Automap Controller
class AutomapController : public Automap::ParamInfo
{
	friend class AutomapDevice;

	public:
		typedef enum Type {ePOT, eBUTTON, eENDLESS, eMOMENTARY};		//!< Controller type (for min/max/step and endless/momentary status)

	private:
		AutomapController();											//!< Constructor (forbidden access)

		ReaperControlType	_type;										//!< Reaper control type
		int					_trackIndex;								//!< Reaper track index (0 = master)
		set<int>			_paramIds;									//!< Automap ParamIds assigned to controller
		Type				_ctrlType;									//!< Controller type
		map<int, int>		_slots;										//!< Controller paramId/slot map
		map<int, float>		_value;										//!< Controller paramId/value map (used for slot controls)

	public:
		AutomapController(ReaperControlType type, int trackIndex, Type ctrlType);	//!< Constructor
		~AutomapController();														//!< Destructor

		int getTrackIndex();														//!< Get track index
		bool isEndless();															//!< Return true if controller is endless

		ReaperControlType getType();												//!< Get control type
		void refreshClient(AutomapReaperClient *client, bool updateInfo = false);	//!< Update Automap client (call SendEvent())

		int getSlot(int paramId);
		void setSlot(int paramId, int slot);
		float getValue(int slot = 0);
		void setValue(float value, int slot = 0);
};

//! \brief Base class for Automap devices
//! \note Max number of parameters in Automap 3.2 is 2048
class AutomapDevice
{
	public:
		typedef enum { eFIRST = -1, eREMOTESL, eREMOTESLCOMPACT, eIPHONE, eNOCTURN, eREMOTESLDYNAMIC, eREMOTESLPADRES, eLAST  } Type;		//!< Device type (for proper mapping)

	private:
		AutomapDevice();															//!< Constructor (forbidden access)

	protected:
		bool _hasGlobalPages;														//!< True if device is using global pages
		map<pair<ReaperControlType,int>, AutomapController*> _ctrlTypeIndexed;		//!< type/trackidx-indexed controllers
		map<int, AutomapController*> _ctrlParamIdIndexed;							//!< paramId-indexed controllers
		set<AutomapController*> _controllers;										//!< Controllers
		int _nbParamsPerPage;														//!< Number parameters per page
		int _nbTracksPerPage;														//!< Number of tracks per "track page"
		int _nbGlobalPages;															//!< Number of global pages
		int _nbFxParamSlots;														//!< Number of FX param slots available
		int _nbTcpFxParamSlots;														//!< Number of TCP/MCP FX param slots available
		int _nbTrackSendSlots;														//!< Number of Track Send slots available
		int _nbTrackHardSendSlots;													//!< Number of Track Hardware Send slots available
		//int _nbAssignableFnSlots;													//!< Number of assignable function slots available
int _nbPresets;
int _currentPresetId;
map<int, const char*> _presetNames;

		AutomapDevice(bool hasGlobalPages);											//!< Constructor

		AutomapController* addController(ReaperControlType type, int trackIndex,
			 AutomapController::Type ctrlType, int paramId, int slot = 0, float value = 0.5f);		//!< Add a controller
		void removeController(ReaperControlType type, int trackIndex, int paramId = -1);			//!< Remove paramId from controller (if last paramId or paramId=-1 : delete)

		void clearControllers();													//!< Clear all controllers
		virtual void initGlobalControllers() = 0;									//!< Create global controllers (if has global pages)
		virtual void initTrackControllers(int nbTracks) = 0;						//!< Init track controllers (to be called each time tracklist changes)

	public:
		virtual ~AutomapDevice();													//!< Destructor

		AutomapController* getController(ReaperControlType type, int trackIndex);	//!< Get controller (type/trackidx-indexed)
		AutomapController* getController(int paramId);								//!< Get controller (paramId-indexed)

		void initControllers(int nbTracks, int preset = -1);							//!< Reinit all controllers

int getNbPresets();
int getCurrentPreset(char name[AUTOMAP_NAME_LENGTH+1]);


		int getNbParams();											//!< Return number of parameters
		int getNbParamsPerPage();									//!< Return number of parameters per page
		int getNbTracksPerPage();									//!< Return number of tracks per page (non-global pages!)
		int getNbGlobalPages();										//!< Return number of global pages
		int getNbFxParamSlots();									//!< Return number of FX param slots available
		int getNbTcpFxParamSlots();									//!< Return number of TCP/MCP FX param slots available
		int getNbTrackSendSlots();									//!< Return number of Track Send slots available
		int getNbTrackHardSendSlots();								//!< Return number of Track Hardware Send slots available
		//int getNbAssignableFnSlots();								//!< Return number of assignable function slots available

		void getVolValueStr(double value, char* valueStr);			//!< Fill Volume string (dB)
		void getPanValueStr(double value, char* valueStr);			//!< Fill Pan string (%)
		void getButtonValueStr(double value, char* valueStr);		//!< Fill Button string (on/off)
		void getTrackSelectValueStr(double value, char* valueStr);	//!< Track Selection Button string ("^"/" ")

		void getSubmenuValueStr(double value, char* valueStr);			//!< Fill Submenu selection string (""/"^")
		void getMcpModeValueStr(double value, char* valueStr);			//!< Fill Track/Mixer view string
		void getSendModeValueStr(int mode, char* valueStr);				//!< Fill Send Mode string
		void getMidiViewModeValueStr(int view, char* valueStr);			//!< Fill Midi View Mode string
		int getMidiViewModeCommand(int view);							//!< Get Midi View Mode command ID
		void getGridValueStr(GridDivision grid, char* valueStr);		//!< Fill Midi Grid string
		void getCcLaneValueStr(int lane, char* valueStr);				//!< Fill CC Lane string
void getAutomationModeValueStr(int mode, char* valueStr);
void getEnvShapeValueStr(int shape, char* valueStr);
int getDefaultEnvShapeCommand(int shape);
int getSelectedEnvShapeCommand(int shape);
};

//! \brief AutomapDevice class for RemoteSL mappings
//! \note Remote (Zero) SL Parameter Ids:\n\n
//!  - 0->7		: Encoders, Page 1\n
//!  - 8->15	: Pots, Page 1\n
//!  - 16->23	: Sliders, Page 1\n
//!  - 24->31	: Left Buttons, top row, Page 1\n
//!  - 32->39	: Left Buttons, second row, Page 1\n
//!  - 40->47	: Right Buttons, top row, Page 1\n
//!  - 48->55	: Encoders, Page 2\n
//!  - etc.\n
class AutomapRemoteSl : public AutomapDevice
{
//	friend class AutomapRemoteSlDynamic;

	protected:
		//! \brief RemoteSL parameter type
		typedef enum {	ePARAM_ENCODER = 0,
						ePARAM_POT = 1,
						ePARAM_SLIDER = 2,
						ePARAM_BUTTON_LEFT1 = 3,
						ePARAM_BUTTON_LEFT2 = 4,
						ePARAM_BUTTON_RIGHT = 5,
						eUNKNOWN
					} ParamType;

		ParamType getParamType(int paramIndex);				//!< Return parameter type

	public:
		AutomapRemoteSl(bool hasGlobalPages);				//!< Constructor
		virtual ~AutomapRemoteSl();							//!< Destructor

	// AutomapDevice compliance:

	private:
		virtual void initGlobalControllers();
		virtual void initTrackControllers(int nbTracks);
};

//! \brief AutomapDevice inherited class for iPhone mappings
//! \note iPhone Parameter Ids:\n\n
//!  - 0->1		: Sliders, Page 1\n
//!  - 2->9		: Buttons, Page 1\n
//!  - 10->11	: Sliders, Page 2\n
//!  - etc.\n
class AutomapiPhone : public AutomapDevice
{
	private:
		//! \brief iPhone parameter type
		typedef enum {	ePARAM_SLIDER,
						ePARAM_BUTTON,
						ePARAM_UNKNOWN
					} ParamType;

		ParamType getParamType(int paramIndex);				//!< Return parameter type

	public:
		AutomapiPhone(bool hasGlobalPages);					//!< Constructor
		virtual ~AutomapiPhone();							//!< Destructor

	// AutomapDevice compliance:

	private:
		virtual void initGlobalControllers();
		virtual void initTrackControllers(int nbTracks);
};

//! \brief AutomapDevice class for Nocturn mappings
//! \note Nocturn Parameter Ids:\n\n
//!  - 0->3		: Left encoders, Page 1\n
//!  - 4->7		: Right encoders, Page 1\n
//!  - 8->11	: Left buttons, Page 1\n
//!  - 12->15	: Right buttons, Page 1\n
//!  - 16->19	: Left encoders, Page 2\n
//!  - etc.\n
class AutomapNocturn : public AutomapDevice
{
	private:
		//! \brief Nocturn parameter type
		typedef enum {	ePARAM_ENCODER = 0,
						ePARAM_BUTTON = 1,
						ePARAM_UNKNOWN
					} ParamType;

		ParamType getParamType(int paramIndex);				//!< Return parameter type

	public:
		AutomapNocturn(bool hasGlobalPages);				//!< Constructor
		virtual ~AutomapNocturn();							//!< Destructor

	// AutomapDevice compliance:

	private:
		virtual void initGlobalControllers();
		virtual void initTrackControllers(int nbTracks);
};

//! \brief AutomapDevice class for RemoteSL Compact mappings
//! \note RemoteSL Compact Parameter Ids:\n\n
//!  - 0->7		: Encoders Row 1, Page 1\n
//!  - 8->15	: Encoders Row 2, Page 1\n
//!  - 16->23	: Encoders Row 3, Page 1\n
//!  - 24->31	: Buttons Row 1, Page 1\n
//!  - 32->39	: Buttons Row 2, Page 1\n
//!  - 40->47	: Buttons Row 3, Page 1\n
//!  - 48->55	: Encoders Row 1, Page 2\n
//!  - etc.\n
class AutomapRemoteSlCompact : public AutomapRemoteSl
{
	private:
		//! \brief RemoteSL Compact parameter type
		typedef enum {	ePARAM_ENCODER1 = 0,
						ePARAM_ENCODER2 = 1,
						ePARAM_ENCODER3 = 2,
						ePARAM_BUTTON1 = 3,
						ePARAM_BUTTON2 = 4,
						ePARAM_BUTTON3 = 5,
						eUNKNOWN
					} ParamType;

		ParamType getParamType(int paramIndex);						//!< Return parameter type

	public:
		AutomapRemoteSlCompact(bool hasGlobalPages);				//!< Constructor
		virtual ~AutomapRemoteSlCompact();							//!< Destructor

	// AutomapDevice compliance:

	private:
		virtual void initTrackControllers(int nbTracks);
};

#endif	// __AUTOMAP_DEVICE_H__
