/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#include "AutomapCSurf.h"

//! \brief Create Control Surface
static IReaperControlSurface *createFunc(const char *type_string, const char *configString, int *errStats)
{
	if(AutomapCSurf::configStringToParameters(configString, AutomapCSurf::_parameters))
	{
		AutomapCSurf::loadConfigFile();
		return new AutomapCSurf(errStats);
	}
	else
		return NULL;
}

//! \brief Control Surface dialog box
static WDL_DLGRET dlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
		{
			if(!hwndDlg)
				return 1;

			AutomapCSurf::Parameters parameters;
			AutomapCSurf::configStringToParameters((const char*)lParam, parameters);

			if(parameters.hasGlobalPages)
				CheckDlgButton(hwndDlg,IDC_CHECK_GLOBALPAGES,BST_CHECKED);
			else
				CheckDlgButton(hwndDlg,IDC_CHECK_GLOBALPAGES,BST_UNCHECKED);

			if(parameters.mcpMode)
				CheckDlgButton(hwndDlg,IDC_CHECK_MCPMODE,BST_CHECKED);
			else
				CheckDlgButton(hwndDlg,IDC_CHECK_MCPMODE,BST_UNCHECKED);

			int firstId = IDC_RADIO_FFWRWD;
			int lastId = IDC_RADIO_CURSOR;
			switch(parameters.ffwRwdMode)
			{
				case AutomapCSurf::Parameters::eFFWRWD_STANDARD			: CheckRadioButton(hwndDlg, firstId, lastId, IDC_RADIO_FFWRWD);				break;
				case AutomapCSurf::Parameters::eFFWRWD_STARTEND			: CheckRadioButton(hwndDlg, firstId, lastId, IDC_RADIO_GOSTARTEND);			break;
				case AutomapCSurf::Parameters::eFFWRWD_NEXTPREVMARKER	: CheckRadioButton(hwndDlg, firstId, lastId, IDC_RADIO_NEXTPREVMARKER);		break;
				case AutomapCSurf::Parameters::eFFWRWD_MOVECURSOR		: CheckRadioButton(hwndDlg, firstId, lastId, IDC_RADIO_CURSOR);				break;
				default													: CheckRadioButton(hwndDlg, firstId, lastId, -1);							break;
			}

			char name[BUFFER_SIZE] = "???";
			for(int i=(int)(AutomapDevice::eFIRST)+1; i<(int)(AutomapDevice::eLAST); i++)
			{
				switch(i)
				{
					case AutomapDevice::eREMOTESL			: strcpy(name, "ReMOTE SL");				break;
					case AutomapDevice::eREMOTESLCOMPACT	: strcpy(name, "ReMOTE SL COMPACT");		break;
					case AutomapDevice::eIPHONE				: strcpy(name, "iPhone");					break;
					case AutomapDevice::eNOCTURN			: strcpy(name, "Nocturn");					break;
					case AutomapDevice::eREMOTESLDYNAMIC	: strcpy(name, "ReMOTE SL (dynamic)");		break;
					case AutomapDevice::eREMOTESLPADRES		: strcpy(name, "ReMOTE SL (Padre's)");		break;
					default :
					break;
				}

				int x = SendDlgItemMessage(hwndDlg,IDC_COMBO_MAPPING,CB_ADDSTRING,0,(LPARAM)name);
				SendDlgItemMessage(hwndDlg,IDC_COMBO_MAPPING,CB_SETITEMDATA,x,i);
		        if(i == parameters.deviceType)
					SendDlgItemMessage(hwndDlg,IDC_COMBO_MAPPING,CB_SETCURSEL,x,0);
			}
		
		}
	    break;

		case WM_USER+1024:
			if (wParam > 1 && lParam)
			{
				if(!hwndDlg)
					return 1;

				char tmp[512];

				AutomapCSurf::Parameters parameters;
				parameters.hasGlobalPages = (IsDlgButtonChecked(hwndDlg,IDC_CHECK_GLOBALPAGES) != 0);
				parameters.mcpMode = (IsDlgButtonChecked(hwndDlg,IDC_CHECK_MCPMODE) != 0);
				
				if(IsDlgButtonChecked(hwndDlg,IDC_RADIO_FFWRWD))
					parameters.ffwRwdMode = AutomapCSurf::Parameters::eFFWRWD_STANDARD;
				else if(IsDlgButtonChecked(hwndDlg,IDC_RADIO_GOSTARTEND))
					parameters.ffwRwdMode = AutomapCSurf::Parameters::eFFWRWD_STARTEND;
				else if(IsDlgButtonChecked(hwndDlg,IDC_RADIO_NEXTPREVMARKER))
					parameters.ffwRwdMode = AutomapCSurf::Parameters::eFFWRWD_NEXTPREVMARKER;
				else if(IsDlgButtonChecked(hwndDlg,IDC_RADIO_CURSOR))
					parameters.ffwRwdMode = AutomapCSurf::Parameters::eFFWRWD_MOVECURSOR;

				int dev = SendDlgItemMessage(hwndDlg,IDC_COMBO_MAPPING,CB_GETCURSEL,0,0);
				if(dev != CB_ERR)
					parameters.deviceType = (AutomapDevice::Type)(SendDlgItemMessage(hwndDlg,IDC_COMBO_MAPPING,CB_GETITEMDATA,dev,0));

				parameters.midiOut = AutomapCSurf::_parameters.midiOut;

				AutomapCSurf::parametersToConfigString(parameters, tmp);
				lstrcpyn((char *)lParam, tmp, wParam);
			}
		break;

		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDC_AUTODETECT:
				{
					AutomapCSurf::_parameters.midiOut = -1;
					int midiPortIn, midiPortOut;
					if(AutomapSLRawMidiClient::detectDeviceMidiPorts(midiPortIn, midiPortOut))
					{
						char msg[BUFFER_SIZE];
						char buf[BUFFER_SIZE];
						strcpy(msg, "You can test ReMOTE SL dynamic mapping! :)\n\n");
						if(GetMIDIInputName(midiPortIn,buf,sizeof(buf)))
							sprintf(msg, "%sMIDI-In\t: %d-%s\n", msg, midiPortIn, buf);
						if(GetMIDIInputName(midiPortOut,buf,sizeof(buf)))
							sprintf(msg, "%sMIDI-Out\t: %d-%s\n", msg, midiPortOut, buf);
						MessageBox(hwndDlg, msg, "ReMOTE SL detected", MB_OK);
						AutomapCSurf::_parameters.midiOut = midiPortOut;
					}
					else
						MessageBox(hwndDlg, "You cannot test ReMOTE SL dynamic mapping! :(", "ReMOTE SL NOT detected", MB_ICONEXCLAMATION | MB_OK);
				}
				break;
				default:
					return DefWindowProc(hwndDlg, uMsg, wParam, lParam);
				break;
			}
		break;
	}

	return 0;
}

//! \brief Set control surface dialog box parameters
static HWND configFunc(const char *type_string, HWND parent, const char *initConfigString)
{
	return CreateDialogParam(g_hInst,MAKEINTRESOURCE(IDD_SURFACEEDIT_AUTOMAP),parent,dlgProc,(LPARAM)initConfigString);
}

//! \brief Control surface registration structure
reaper_csurf_reg_t csurf_automap_reg = 
{
	"Automap",
	"Novation Automap",
	createFunc,
	configFunc,
};
