/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#include "AutomapExtraMappings.h"

AutomapRemoteSlDynamic::AutomapRemoteSlDynamic(bool hasGlobalPages)
: AutomapRemoteSl(hasGlobalPages)
{
	_submenu = eSUBMENU_TCPFX;

	//! \bug Careful using presets with current AutomapServer.exe page delete crash!
	_nbPresets = eNBPRESETS;
	_currentPresetId = ePRESET_MAINCONTROL;
	_presetNames[ePRESET_MAINCONTROL] = "Main controls";
	_presetNames[ePRESET_TRACKEDIT] = "Track/Item editor";
	_presetNames[ePRESET_ENVEDIT] = "Track/Envelope editor";
	_presetNames[ePRESET_MIDIEDIT] = "MIDI editor";
	_presetNames[ePRESET_ASSIGNABLE] = "Assignable controls";
}

AutomapRemoteSlDynamic::~AutomapRemoteSlDynamic()
{
}

int AutomapRemoteSlDynamic::getSubmenu()
{
	return (int)(_submenu);
}

void AutomapRemoteSlDynamic::setSubmenu(int submenu)
{
	_submenu = (Submenu)(submenu);
}

int AutomapRemoteSlDynamic::val2Submenu(float value)
{
	return (int)(value * (float)(getNbSubmenus()-1));
}

float AutomapRemoteSlDynamic::submenu2Val(int submenu)
{
	return (float)(submenu) / (float)(getNbSubmenus()-1);
}

void AutomapRemoteSlDynamic::getSubmenuName(int submenu, char* name)
{
	switch(submenu)
	{
		case eSUBMENU_FX:						sprintf(name, "FX");			break;
		case eSUBMENU_TCPFX:					sprintf(name, "T/MCP FX");		break;
		case eSUBMENU_TRACKSEND_VOL:			sprintf(name, "Send Vol");		break;
		case eSUBMENU_TRACKSEND_PAN:			sprintf(name, "Send Pan");		break;
		case eSUBMENU_TRACKSEND_MUTE:			sprintf(name, "Send Mute");		break;
		case eSUBMENU_TRACKSEND_MODE:			sprintf(name, "Send Mode");		break;
		case eSUBMENU_TRACKHARDSEND_VOL:		sprintf(name, "HSend Vol");		break;
		case eSUBMENU_TRACKHARDSEND_PAN:		sprintf(name, "HSend Pan");		break;
		case eSUBMENU_TRACKHARDSEND_MUTE:		sprintf(name, "HSend Mute");	break;
		case eSUBMENU_TRACKHARDSEND_MODE:		sprintf(name, "HSend Mode");	break;
		default:								sprintf(name, "???");			break;
	}
}

void AutomapRemoteSlDynamic::initGlobalControllers()
{
	_nbGlobalPages = 3;
	//_nbGlobalPages = 5;
	_nbGlobalPages = 1;
	_nbFxParamSlots = 8;
	_nbTcpFxParamSlots = 8;
	_nbTrackSendSlots = 8;
	_nbTrackHardSendSlots  = 8;

	if(_hasGlobalPages)
	{
		//initMainControlPage(0);
		//initTrackEditControlPage(1);
		//initEnvEditControlPage(2);
		//initMidiEditControlPage(3);
		//initAssignableControlPage(4);

		switch(_currentPresetId)
		{
			case ePRESET_MAINCONTROL	: initMainControlPage(0);			break;
			case ePRESET_TRACKEDIT		: initTrackEditControlPage(0);		break;
			case ePRESET_ENVEDIT		: initEnvEditControlPage(0);		break;
			case ePRESET_MIDIEDIT		: initMidiEditControlPage(0);		break;
			case ePRESET_ASSIGNABLE		: initAssignableControlPage(0);		break;
			default:
			break;
		}

		//// Global Page 1: main controls

//		int pageOffset = 0;
//
//		addController(eCTRL_LOOP_SETSTART,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
//		addController(eCTRL_LOOP_SETEND,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
//		addController(eCTRL_GOSTART,				-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
//		addController(eCTRL_GOEND,					-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
//		addController(eCTRL_MARKER_PREV,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
//		addController(eCTRL_MARKER_NEXT,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
//		addController(eCTRL_INSERT_MARKER,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
//		addController(eCTRL_DELETE_MARKER,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);
//
//		addController(eCTRL_GRID_LINES,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
//		addController(eCTRL_SNAP,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
//		addController(eCTRL_GROUPING_OVERRIDE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
//		addController(eCTRL_METRONOME,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
//		addController(eCTRL_INSERT_MARKERRGN,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
//		addController(eCTRL_CLEARRECARM,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
//		addController(eCTRL_MCPMODE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);
//
//		addController(eCTRL_GRID_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
//		addController(eCTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
//		addController(eCTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
//		addController(eCTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
//		addController(eCTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
//		addController(eCTRL_PREVNEXTMARKER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
//		addController(eCTRL_FFWRWD_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
//		addController(eCTRL_SCRUB_JOG_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
//
//		addController(eCTRL_HZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);
//		addController(eCTRL_VZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+1);
//		addController(eCTRL_HSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+2);
//		addController(eCTRL_VSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+3);
//		addController(eCTRL_METRONOME_VOLUME,		-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+5);
//		addController(eCTRL_VOL,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+pageOffset+6);
//		addController(eCTRL_PAN,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+pageOffset+7);
//
////addController(eCTRL_SWS_TRACKLIST,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
//		addController(eCTRL_NAVIGATOR_WINDOW,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
//		addController(eCTRL_MIXER_VISIBLE,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
//		addController(eCTRL_TRACK_GROUPING_MATRIX,	-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
//		addController(eCTRL_ROUTING_MATRIX,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
//		addController(eCTRL_PERFORMANCE_METER,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);

		//// Global Page 2: main track/item edit

		//pageOffset = AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

		//addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
		//addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
		//addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
		//addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
		//addController(eCTRL_CURSEL_PHASE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
		//addController(eCTRL_CURSEL_VIEW_IO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
		//addController(eCTRL_CURSEL_VIEW_ENVELOPES,	-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
		//addController(eCTRL_CURSEL_VIEW_FX,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

		//addController(eCTRL_GRID_WHEEL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
		//addController(eCTRL_TRACKSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
		//addController(eCTRL_ITEMSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
		//addController(eCTRL_TAKESELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
		//addController(eCTRL_ITEM_PITCH_OCTAVE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
		//addController(eCTRL_ITEM_PITCH_SEMITONE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
		//addController(eCTRL_ITEM_PITCH_ONECENT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

		//addController(eCTRL_CURSEL_VIEW_GROUPING,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
		//addController(eCTRL_TRACK_FREEPOS,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
		//addController(eCTRL_ITEM_SELECTALL_INSELTRACK,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
		//addController(eCTRL_ITEM_SELECTALL_INTIMESEL,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
		//addController(eCTRL_ITEM_SELECTALL_INGROUP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
		//addController(eCTRL_ITEM_PROPERTIES,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
		//addController(eCTRL_ITEM_GROUP,						-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
		//addController(eCTRL_ITEM_REMOVE_FROMGROUP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

		//addController(eCTRL_CURSEL_VOL,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);
		//addController(eCTRL_CURSEL_PAN,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+1);
		//addController(eCTRL_HZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+4);
		//addController(eCTRL_VZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+5);
		//addController(eCTRL_HSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+6);
		//addController(eCTRL_VSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+7);

		//addController(eCTRL_ITEM_MUTE,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
		//addController(eCTRL_ITEM_LOOP_SOURCE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
		//addController(eCTRL_ITEM_LOOP_SECTION,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
		//addController(eCTRL_ITEM_MOVE_LEFT,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
		//addController(eCTRL_ITEM_MOVE_RIGHT,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
		//addController(eCTRL_TAKE_REVERSE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
		//addController(eCTRL_TAKE_ACTIVE_CROP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
		//addController(eCTRL_TAKE_ACTIVE_DELETE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

		//// Global Page 3: main track/envelope edit

		//pageOffset = 2*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

		//addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
		//addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
		//addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
		//addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
		//addController(eCTRL_CURSEL_PHASE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
		//addController(eCTRL_CURSEL_VIEW_IO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
		//addController(eCTRL_CURSEL_VIEW_ENVELOPES,	-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
		//addController(eCTRL_CURSEL_VIEW_FX,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

		//addController(eCTRL_GRID_WHEEL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
		//addController(eCTRL_TRACKSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
		//addController(eCTRL_FFWRWD_WHEEL,					-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
		//addController(eCTRL_SCRUB_JOG_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
		//addController(eCTRL_AUTOMATION_ACTIVE_FADER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

		//addController(eCTRL_AUTOMATION_ACTIVE_FADER_POT,	-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);

		//addController(eCTRL_AUTOMATION_ACTIVE_FADER_POT,	-1, AutomapController::ePOT,		8*(int)(ePARAM_POT)+pageOffset);

		//addController(eCTRL_CURSEL_AUTOMATION_MODE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
		//addController(eCTRL_ENVELOPE_DEFAULT_SHAPE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
		//addController(eCTRL_SHOW_ALL_ACTIVE_ENVELOPES,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
		//addController(eCTRL_ENVELOPE_ALLVISIBLE_LANE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
		//addController(eCTRL_ENVELOPE_BYPASS,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
		//addController(eCTRL_ENVELOPE_RECARM,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
		//addController(eCTRL_ENVELOPE_TOGGLE_DISPLAY,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);

		//addController(eCTRL_ENVELOPE_INSERT_POINT,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
		//addController(eCTRL_ENVELOPE_DELETE_POINTS,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
		//addController(eCTRL_ENVELOPE_SELECTED_SHAPE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
		//addController(eCTRL_ENVELOPE_CLEAR,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
		//addController(eCTRL_ENVELOPE_VISIBLE_PREFXVOL,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
		//addController(eCTRL_ENVELOPE_VISIBLE_VOLUME,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
		//addController(eCTRL_ENVELOPE_VISIBLE_PAN,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);

/*
		// Global Page 1

		addController(eCTRL_LOOP_SETSTART,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1));
		addController(eCTRL_LOOP_SETEND,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+1);
		addController(eCTRL_GOSTART,				-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+2);
		addController(eCTRL_GOEND,					-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+3);
		addController(eCTRL_MARKER_PREV,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+4);
		addController(eCTRL_MARKER_NEXT,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+5);
		addController(eCTRL_INSERT_MARKER,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+6);
		addController(eCTRL_DELETE_MARKER,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+7);
		//addController(eCTRL_INSERT_MARKERRGN,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT1)+7);

		addController(eCTRL_MCPMODE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2));
		addController(eCTRL_CLEARRECARM,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+1);
		addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+4);
		addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+5);
		addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+6);
		addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+7);

		addController(eCTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER));
		addController(eCTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+1);
		addController(eCTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+2);
		addController(eCTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+3);
		addController(eCTRL_PREVNEXTMARKER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+4);
		addController(eCTRL_FFWRWD_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+5);
		addController(eCTRL_SCRUB_JOG_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+6);
		addController(eCTRL_TRACKSELECT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+7);

		addController(eCTRL_HZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER));
		addController(eCTRL_VZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+1);
		addController(eCTRL_HSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+2);
		addController(eCTRL_VSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+3);
		addController(eCTRL_CURSEL_VOL,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+4);
		addController(eCTRL_CURSEL_PAN,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+5);
		addController(eCTRL_VOL,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+6);
		addController(eCTRL_PAN,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+7);

		for(int slot=0; slot<8; slot++)
			addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+slot, slot);

		for(int slot=0; slot<8; slot++)
			addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_POT)+slot, slot);
*/
		//// Global Page 4: MIDI Editor

		//pageOffset = 3*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

		//addController(eMAINMIDICTRL_INSERTITEM,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
		//addController(eMAINMIDICTRL_OPENITEMS,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
		//addController(eMIDICTRL_NEXTSECONDARY,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
		//addController(eMIDICTRL_TOGGLE_VIEWMODE,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
		//addController(eMIDICTRL_TOGGLE_GRID,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
		//addController(eMIDICTRL_TOGGLE_SNAP,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
		//addController(eMIDICTRL_FIT_LOOP,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
		//addController(eMIDICTRL_FIT_CONTENT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

		//addController(eMIDICTRL_NOTE_SELECTPREV,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
		//addController(eMIDICTRL_NOTE_SELECTNEXT,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
		//addController(eMIDICTRL_NOTE_VALUE_SELPREV,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
		//addController(eMIDICTRL_NOTE_VALUE_SELNEXT,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
		//addController(eMIDICTRL_TRANSPOSE_DEC1,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
		//addController(eMIDICTRL_TRANSPOSE_INC1,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
		//addController(eMIDICTRL_NOTE_MOVELEFT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
		//addController(eMIDICTRL_NOTE_MOVERIGHT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

		//addController(eMIDICTRL_MOVE_UP,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
		//addController(eMIDICTRL_MOVE_DOWN,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
		//addController(eMIDICTRL_MOVE_LEFT,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
		//addController(eMIDICTRL_MOVE_RIGHT,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
		//addController(eMIDICTRL_INSERTNOTE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
		//addController(eMIDICTRL_DELETENOTE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
		//addController(eMIDICTRL_NOTEPROPERTIES,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
		//addController(eMIDICTRL_HUMANIZE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

		//addController(eMIDICTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
		//addController(eMIDICTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
		//addController(eMIDICTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
		//addController(eMIDICTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
		//addController(eMIDICTRL_GRID_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
		////addController(eMIDICTRL_VMOVE_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
		////addController(eMIDICTRL_HMOVE_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
		//addController(eMIDICTRL_VELOCITY_COARSE_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
		//addController(eMIDICTRL_VELOCITY_FINE_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
		//addController(eMIDICTRL_NOTE_LENGTH_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

		//addController(eMIDICTRL_CCLANE_POT,	-1, AutomapController::ePOT,	8*(int)(ePARAM_POT)+pageOffset);

		//// Global Page 5: Assignable Controls

		//pageOffset = 4*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+slot, slot);
		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+slot, slot+8);
		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+slot, slot+16);
		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_POT)+pageOffset+slot, slot);
		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_SLIDER)+pageOffset+slot, slot+8);
		//for(int slot=0; slot<8; slot++)
		//	addController(eCTRL_ASSIGNABLEFNENC,	-1, AutomapController::eENDLESS, 8*(int)(ePARAM_ENCODER)+pageOffset+slot, slot);
	}
}

void AutomapRemoteSlDynamic::initMainControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	addController(eCTRL_MCPMODE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
	addController(eCTRL_SNAP,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
	addController(eCTRL_GROUPING_OVERRIDE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
	addController(eCTRL_METRONOME,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
	addController(eCTRL_GRID_LINES,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eCTRL_NAVIGATOR_WINDOW,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
	addController(eCTRL_PERFORMANCE_METER,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
	addController(eCTRL_UNDO_HISTORY,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);
	//addController(eCTRL_CLEARRECARM,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);

	addController(eCTRL_LOOP_SETSTART,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
	addController(eCTRL_LOOP_SETEND,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
	addController(eCTRL_GOSTART,				-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
	addController(eCTRL_GOEND,					-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
	//addController(eCTRL_MARKER_PREV,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	//addController(eCTRL_MARKER_NEXT,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eCTRL_INSERT_MARKER,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	addController(eCTRL_DELETE_MARKER,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eCTRL_INSERT_MARKERRGN,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
	addController(eCTRL_SWS_ZOOM_TO_TRACKTIMESELECT,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

	addController(eCTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	addController(eCTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	addController(eCTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	addController(eCTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eCTRL_GRID_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
	addController(eCTRL_PREVNEXTMARKER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	//addController(eCTRL_FFWRWD_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	addController(eCTRL_MOVE_CURSOR_GRID_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	//addController(eCTRL_SCRUB_JOG_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
	addController(eCTRL_SPEED_DIAL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eCTRL_HSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);
	addController(eCTRL_VSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+1);
	addController(eCTRL_HZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+2);
	addController(eCTRL_VZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+3);
	addController(eCTRL_SOLO_INFRONT_ADJUST_POT,		-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+4);
	addController(eCTRL_METRONOME_VOLUME,		-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+5);
	addController(eCTRL_VOL,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+pageOffset+6);
	addController(eCTRL_PAN,					0, AutomapController::ePOT,			8*(int)(ePARAM_SLIDER)+pageOffset+7);

	addController(eCTRL_MIXER_VISIBLE,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
	addController(eCTRL_TRACK_GROUPING_MATRIX,	-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
	addController(eCTRL_ROUTING_MATRIX,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
	addController(eCTRL_SWS_TRACKLIST,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	addController(eCTRL_SWS_MARKERLIST,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
	addController(eCTRL_SWS_SNAPSHOTLIST,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
	addController(eCTRL_SWS_NEW_SNAPSHOT,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
	addController(eCTRL_SWS_SAVE_SNAPSHOT,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);
	//addController(eCTRL_NAVIGATOR_WINDOW,		-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	//addController(eCTRL_UNDO_HISTORY,			-1, AutomapController::eMOMENTARY,		8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_POT)+pageOffset+slot, slot);
}

void AutomapRemoteSlDynamic::initTrackEditControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
	addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
	addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
	addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
	addController(eCTRL_CURSEL_PHASE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eCTRL_CURSEL_VIEW_IO,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
	addController(eCTRL_CURSEL_VIEW_ENVELOPES,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
	addController(eCTRL_CURSEL_VIEW_FX,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

	addController(eCTRL_GRID_WHEEL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	addController(eCTRL_TRACKSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	addController(eCTRL_ITEMSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	addController(eCTRL_TAKESELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eCTRL_ITEM_PITCH_OCTAVE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	addController(eCTRL_ITEM_PITCH_SEMITONE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	addController(eCTRL_ITEM_PITCH_ONECENT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eCTRL_CURSEL_VIEW_GROUPING,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
	addController(eCTRL_TRACK_FREEPOS,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
	addController(eCTRL_ITEM_SELECTALL_INSELTRACK,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
	addController(eCTRL_ITEM_SELECTALL_INTIMESEL,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
	addController(eCTRL_ITEM_SELECTALL_INGROUP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	addController(eCTRL_ITEM_PROPERTIES,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eCTRL_ITEM_GROUP,						-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
	addController(eCTRL_ITEM_REMOVE_FROMGROUP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

	addController(eCTRL_CURSEL_VOL,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);
	addController(eCTRL_CURSEL_PAN,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+1);
	addController(eCTRL_HSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+4);
	addController(eCTRL_VSCROLL_POT,			-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+5);
	addController(eCTRL_HZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+6);
	addController(eCTRL_VZOOM_POT,				-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+7);

	addController(eCTRL_ITEM_MUTE,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
	addController(eCTRL_ITEM_LOOP_SOURCE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
	addController(eCTRL_ITEM_LOOP_SECTION,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
	addController(eCTRL_ITEM_MOVE_LEFT,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	addController(eCTRL_ITEM_MOVE_RIGHT,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
	addController(eCTRL_TAKE_REVERSE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
	addController(eCTRL_TAKE_ACTIVE_CROP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
	addController(eCTRL_TAKE_ACTIVE_DELETE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

for(int slot=0; slot<8; slot++)
	addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_POT)+pageOffset+slot, slot);
}

void AutomapRemoteSlDynamic::initEnvEditControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
	addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
	addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
	addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
	addController(eCTRL_CURSEL_PHASE,			-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eCTRL_CURSEL_VIEW_IO,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
	addController(eCTRL_CURSEL_VIEW_ENVELOPES,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
	addController(eCTRL_CURSEL_VIEW_FX,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

	//addController(eCTRL_GRID_WHEEL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	//addController(eCTRL_TRACKSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	//addController(eCTRL_FFWRWD_WHEEL,					-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	//addController(eCTRL_SCRUB_JOG_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	//addController(eCTRL_AUTOMATION_ACTIVE_FADER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eCTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	addController(eCTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	addController(eCTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	addController(eCTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eCTRL_GRID_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
	addController(eCTRL_MOVE_CURSOR_GRID_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	addController(eCTRL_AUTOMATION_ACTIVE_FADER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	addController(eCTRL_TRACKSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eCTRL_AUTOMATION_ACTIVE_FADER_POT,	-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);

	addController(eCTRL_AUTOMATION_ACTIVE_FADER_POT,	-1, AutomapController::ePOT,		8*(int)(ePARAM_POT)+pageOffset);

	addController(eCTRL_SWS_ZOOM_TO_TRACKTIMESELECT,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
	addController(eCTRL_CURSEL_AUTOMATION_MODE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
	addController(eCTRL_ENVELOPE_DEFAULT_SHAPE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
	addController(eCTRL_SHOW_ALL_ACTIVE_ENVELOPES,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
	addController(eCTRL_ENVELOPE_ALLVISIBLE_LANE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	addController(eCTRL_ENVELOPE_BYPASS,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eCTRL_ENVELOPE_RECARM,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
	addController(eCTRL_ENVELOPE_TOGGLE_DISPLAY,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

	addController(eCTRL_ENVELOPE_INSERT_POINT,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
	addController(eCTRL_ENVELOPE_DELETE_POINTS,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
	addController(eCTRL_ENVELOPE_SELECTED_SHAPE,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
	addController(eCTRL_ENVELOPE_CLEAR,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	addController(eCTRL_ENVELOPE_VISIBLE_PREFXVOL,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
	addController(eCTRL_ENVELOPE_VISIBLE_PREFXPAN,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
	addController(eCTRL_ENVELOPE_VISIBLE_VOLUME,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
	addController(eCTRL_ENVELOPE_VISIBLE_PAN,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);
}

void AutomapRemoteSlDynamic::initMidiEditControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	addController(eMAINMIDICTRL_INSERTITEM,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
	addController(eMAINMIDICTRL_OPENITEMS,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
	addController(eMIDICTRL_NEXTSECONDARY,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
	addController(eMIDICTRL_TOGGLE_VIEWMODE,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
	addController(eMIDICTRL_TOGGLE_GRID,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eMIDICTRL_TOGGLE_SNAP,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
	addController(eMIDICTRL_FIT_LOOP,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
	addController(eMIDICTRL_FIT_CONTENT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

	addController(eMIDICTRL_NOTE_SELECTPREV,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
	addController(eMIDICTRL_NOTE_SELECTNEXT,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
	addController(eMIDICTRL_NOTE_VALUE_SELPREV,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
	addController(eMIDICTRL_NOTE_VALUE_SELNEXT,	-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
	addController(eMIDICTRL_TRANSPOSE_DEC1,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	addController(eMIDICTRL_TRANSPOSE_INC1,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eMIDICTRL_NOTE_MOVELEFT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
	addController(eMIDICTRL_NOTE_MOVERIGHT,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

	addController(eMIDICTRL_MOVE_UP,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
	addController(eMIDICTRL_MOVE_DOWN,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
	addController(eMIDICTRL_MOVE_LEFT,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
	addController(eMIDICTRL_MOVE_RIGHT,			-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	addController(eMIDICTRL_INSERTNOTE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
	addController(eMIDICTRL_DELETENOTE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
	addController(eMIDICTRL_NOTEPROPERTIES,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
	addController(eMIDICTRL_HUMANIZE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

	addController(eMIDICTRL_HSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	addController(eMIDICTRL_VSCROLL_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	addController(eMIDICTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	addController(eMIDICTRL_VZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eMIDICTRL_GRID_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
	//addController(eMIDICTRL_VMOVE_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
	//addController(eMIDICTRL_HMOVE_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	addController(eMIDICTRL_VELOCITY_COARSE_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	addController(eMIDICTRL_VELOCITY_FINE_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
	addController(eMIDICTRL_NOTE_LENGTH_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eMIDICTRL_CCLANE_POT,	-1, AutomapController::ePOT,	8*(int)(ePARAM_POT)+pageOffset);
}

void AutomapRemoteSlDynamic::initAssignableControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+slot, slot);
	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+slot, slot+8);
	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY, 8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+slot, slot+16);
	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_POT)+pageOffset+slot, slot);
	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFNPOT,	-1, AutomapController::ePOT, 8*(int)(ePARAM_SLIDER)+pageOffset+slot, slot+8);
	for(int slot=0; slot<8; slot++)
		addController(eCTRL_ASSIGNABLEFNENC,	-1, AutomapController::eENDLESS, 8*(int)(ePARAM_ENCODER)+pageOffset+slot, slot);
}

void AutomapRemoteSlDynamic::initTrackControllers(int nbTracks)
{
//if(_currentPresetId != ePRESET_MIXER)
//return;

	int nbPages = nbTracks/AUTOMAP_REMOTESL_NBTRACKS_PER_PAGE + 1;

	for(int page=0; page<nbPages; page++)
	{
		int pageOffset = page*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;
		if(_hasGlobalPages)
			pageOffset += _nbGlobalPages*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;
		for(int trackOffset=0; trackOffset<8; trackOffset++)
		{
			int track = page*AUTOMAP_REMOTESL_NBTRACKS_PER_PAGE + trackOffset + 1;
			if(track<=nbTracks)
			{
				if(trackOffset==0)	// Global controllers
				{
					addController(eCTRL_CURSEL_SOLO,			-1, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
					addController(eCTRL_CURSEL_MUTE,			-1, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
					addController(eCTRL_CURSEL_ARM,				-1, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
					addController(eCTRL_CURSEL_FXBYPASS,		-1, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
					addController(eCTRL_CURSEL_PHASE,			-1, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
					addController(eCTRL_CURSEL_VIEW_IO,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
					addController(eCTRL_CURSEL_VIEW_ENVELOPES,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
					addController(eCTRL_CURSEL_VIEW_FX,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

					addController(eCTRL_CURSEL_NAME,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);

					//addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4, 0);
					//addController(eCTRL_ASSIGNABLEFN,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5, 1);
					//addController(eCTRL_INSERT_MARKER,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
					//addController(eCTRL_MCPMODE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

					addController(eCTRL_CURSEL_PAN,				-1, AutomapController::ePOT,		8*(int)(ePARAM_ENCODER)+pageOffset);
					addController(eCTRL_SUBMENUSELECT_WHEEL,	-1, AutomapController::ePOT,		8*(int)(ePARAM_ENCODER)+pageOffset+1);
					//addController(eCTRL_HZOOM_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
					//addController(eCTRL_ARROW_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
					//addController(eCTRL_PREVNEXTMARKER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+6);
					//addController(eCTRL_FFWRWD_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
//for(int slot=0; slot<4; slot++)
//	addController(eCTRL_ASSIGNABLEFNENC,	-1, AutomapController::eENDLESS, 8*(int)(ePARAM_ENCODER)+pageOffset+4+slot, slot);

					ReaperControlType controlType = eCTRL_FXPARAM;

					switch(_submenu)
					{
						case eSUBMENU_FX :
							addController(eCTRL_CURSEL_FXSELECT_WHEEL,			-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							addController(eCTRL_CURSEL_FXPARAMSELECT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
							controlType = eCTRL_FXPARAM;
						break;

						case eSUBMENU_TCPFX :
							addController(eCTRL_CURSEL_TCPFXPARAMSELECT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TCPFXPARAM;
							for(int slot=0; slot<4; slot++)
								addController(eCTRL_TCPFXPARAM_ENC,	-1, AutomapController::eENDLESS, 8*(int)(ePARAM_ENCODER)+pageOffset+4+slot, slot);
						break;

						case eSUBMENU_TRACKSEND_VOL :
							addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKSEND_VOL;
						break;

						case eSUBMENU_TRACKSEND_PAN :
							addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKSEND_PAN;
						break;

						case eSUBMENU_TRACKSEND_MUTE :
							addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKSEND_MUTE_POT;
						break;

						case eSUBMENU_TRACKSEND_MODE :
							addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKSEND_MODE_POT;
						break;

						case eSUBMENU_TRACKHARDSEND_VOL :
							addController(eCTRL_CURSEL_TRACKHARDSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKHARDSEND_VOL;
						break;

						case eSUBMENU_TRACKHARDSEND_PAN :
							addController(eCTRL_CURSEL_TRACKHARDSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKHARDSEND_PAN;
						break;

						case eSUBMENU_TRACKHARDSEND_MUTE :
							addController(eCTRL_CURSEL_TRACKHARDSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKHARDSEND_MUTE_POT;
						break;

						case eSUBMENU_TRACKHARDSEND_MODE :
							addController(eCTRL_CURSEL_TRACKHARDSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
							controlType = eCTRL_TRACKHARDSEND_MODE_POT;
						break;

						default :
						break;
					}

					for(int slot=0; slot<8; slot++)
						addController(controlType,	-1, AutomapController::ePOT,		8*(int)(ePARAM_POT)+pageOffset+slot, slot);

				}

				addController(eCTRL_VOL,			track, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+trackOffset+pageOffset);
				addController(eCTRL_TRACKSELECT,	track, AutomapController::eBUTTON,	8*(int)(ePARAM_BUTTON_RIGHT)+trackOffset+pageOffset);
			}
		}
	}
}


AutomapRemoteSlPadres::AutomapRemoteSlPadres(bool hasGlobalPages)
: AutomapRemoteSlDynamic(hasGlobalPages)
{
	_submenu = eSUBMENU_TCPFX;

	_nbPresets = eNBPRESETS;
	_currentPresetId = ePRESET_PADRESPAGE;
	_presetNames[ePRESET_PADRESPAGE] = "Padre's favorites";
	_presetNames[ePRESET_MAINCONTROL] = "Main controls";
	_presetNames[ePRESET_TRACKEDIT] = "Track/Item editor";
	_presetNames[ePRESET_ENVEDIT] = "Track/Envelope editor";
	_presetNames[ePRESET_MIDIEDIT] = "MIDI editor";
	_presetNames[ePRESET_ASSIGNABLE] = "Assignable controls";
}

AutomapRemoteSlPadres::~AutomapRemoteSlPadres()
{
}

void AutomapRemoteSlPadres::initGlobalControllers()
{
	_nbGlobalPages = 1;
	_nbFxParamSlots = 8;
	_nbTcpFxParamSlots = 8;
	_nbTrackSendSlots = 8;
	_nbTrackHardSendSlots  = 8;

	if(_hasGlobalPages)
	{
		switch(_currentPresetId)
		{
			case ePRESET_MAINCONTROL	: initMainControlPage(0);			break;
			case ePRESET_TRACKEDIT		: initTrackEditControlPage(0);		break;
			case ePRESET_ENVEDIT		: initEnvEditControlPage(0);		break;
			case ePRESET_MIDIEDIT		: initMidiEditControlPage(0);		break;
			case ePRESET_ASSIGNABLE		: initAssignableControlPage(0);		break;
			case ePRESET_PADRESPAGE		: initPadresControlPage(0);			break;
			default:
			break;
		}
	}
}

void AutomapRemoteSlPadres::initPadresControlPage(int pageId)
{
	int pageOffset = pageId*AUTOMAP_REMOTESL_NBPARAMS_PER_PAGE;

	addController(eCTRL_SNAP,							-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset);
	addController(eCTRL_METRONOME,						-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+1);
	addController(eCTRL_GROUPING_OVERRIDE,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+2);
	addController(eCTRL_MIXER_VISIBLE,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+3);
//	addController(eCTRL_SHOW_ACTIONLIST,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eCTRL_SWS_TRACKLIST,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+4);
	addController(eCTRL_UNDO_HISTORY,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+5);
	addController(eCTRL_SWS_SNAPSHOTLIST,				-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+6);
	addController(eCTRL_SWS_ZOOM_TO_TRACKTIMESELECT,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT1)+pageOffset+7);

	addController(eCTRL_GRID_WHEEL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset);
	addController(eCTRL_ITEM_PITCH_SEMITONE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+1);
	addController(eMIDICTRL_GRID_WHEEL,					-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	//addController(eMIDICTRL_VELOCITY_COARSE_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eMIDICTRL_VELOCITY_FINE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	addController(eCTRL_SPEED_DIAL,						-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+4);
	addController(eCTRL_AUTOMATION_ACTIVE_FADER_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);

	addController(eCTRL_CURSEL_SOLO,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset);
	addController(eCTRL_CURSEL_MUTE,				-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+1);
	addController(eCTRL_CURSEL_ARM,					-1, AutomapController::eBUTTON,		8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+2);
	addController(eCTRL_CURSEL_VIEW_GROUPING,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+3);
	addController(eCTRL_SHOW_ALL_ACTIVE_ENVELOPES,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+4);
	addController(eCTRL_ENVELOPE_ALLVISIBLE_LANE,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+5);
	addController(eCTRL_ENVELOPE_BYPASS,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+6);
	addController(eCTRL_ENVELOPE_SELECTED_SHAPE,	-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_LEFT2)+pageOffset+7);

	addController(eCTRL_HSCROLL_POT,					-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset);
	addController(eCTRL_VSCROLL_POT,					-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+1);
	addController(eCTRL_HZOOM_POT,						-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+2);
	addController(eCTRL_VZOOM_POT,						-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+3);
	addController(eCTRL_CURSEL_VOL,						-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+4);
	addController(eCTRL_CURSEL_PAN,						-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+5);
	addController(eCTRL_AUTOMATION_ACTIVE_FADER_POT,	-1, AutomapController::ePOT,		8*(int)(ePARAM_SLIDER)+pageOffset+6);

	addController(eCTRL_ITEM_MUTE,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset);
	addController(eCTRL_ITEM_GLUED_SELECTED,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+1);
	addController(eCTRL_ITEM_GROUP,					-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+2);
	addController(eCTRL_ITEM_REMOVE_FROMGROUP,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+3);
	addController(eCTRL_ITEM_SELECTALL_INGROUP,		-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+4);
	addController(eCTRL_ITEM_PROPERTIES,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+5);
	addController(eCTRL_TAKE_ACTIVE_CROP,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+6);
	addController(eCTRL_TAKE_ACTIVE_DELETE,			-1, AutomapController::eMOMENTARY,	8*(int)(ePARAM_BUTTON_RIGHT)+pageOffset+7);

	//addController(eCTRL_ITEMSELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+2);
	//addController(eCTRL_TAKESELECT_WHEEL,				-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+3);
	//addController(eCTRL_ITEM_PITCH_OCTAVE_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+5);
	//addController(eCTRL_ITEM_PITCH_ONECENT_WHEEL,		-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);

	addController(eCTRL_SUBMENUSELECT_WHEEL,	-1, AutomapController::ePOT,		8*(int)(ePARAM_ENCODER)+pageOffset+6);
	ReaperControlType controlType = eCTRL_TCPFXPARAM;

	switch(_submenu)
	{
		case eSUBMENU_TCPFX :
			addController(eCTRL_CURSEL_TCPFXPARAMSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
			controlType = eCTRL_TCPFXPARAM;
		break;

		case eSUBMENU_TRACKSEND_VOL :
			addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
			controlType = eCTRL_TRACKSEND_VOL;
		break;

		case eSUBMENU_TRACKSEND_PAN :
			addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
			controlType = eCTRL_TRACKSEND_PAN;
		break;

		case eSUBMENU_TRACKSEND_MUTE :
			addController(eCTRL_CURSEL_TRACKSENDSELECT_WHEEL,	-1, AutomapController::eENDLESS,	8*(int)(ePARAM_ENCODER)+pageOffset+7);
			controlType = eCTRL_TRACKSEND_MUTE_POT;
		break;

		default :
			return;
		break;
	}

	for(int slot=0; slot<8; slot++)
		addController(controlType,	-1, AutomapController::ePOT,		8*(int)(ePARAM_POT)+pageOffset+slot, slot);
}
