/*
** Novation Automap Control Surface for Cockos Reaper
** Copyright (C) 2009 Padre P.C. (Atrabilis Sunrise / P.C.M. / info@p-c-m.org)
** License: LGPL.
**
** This software is provided 'as-is', without any express or implied
** warranty.  In no event will the authors be held liable for any damages
** arising from the use of this software.
**
** Permission is granted to anyone to use this software for any purpose,
** including commercial applications, and to alter it and redistribute it
** freely, subject to the following restrictions:
**
** 1. The origin of this software must not be misrepresented; you must not
**    claim that you wrote the original software. If you use this software
**    in a product, an acknowledgment in the product documentation would be
**    appreciated but is not required.
** 2. Altered source versions must be plainly marked as such, and must not be
**    misrepresented as being the original software.
** 3. This notice may not be removed or altered from any source distribution.
**
** Notes: the C++ interfaces used require MSVC on win32, or at least the MSVC-compatible C++ ABI. Sorry, mingw users :(
**
*/

#ifndef __AUTOMAP_MAPPING_MODS_H__
#define __AUTOMAP_MAPPING_MODS_H__

#include "AutomapDevice.h"

//* General:
//	!SWS: Show Tracklist
//
//* Item edition:
//	Item: Fade items in to cursor
//	Item: Fade items out to cursor
//	Item: Glue selected items

//Item Properties: Lock selected items
//Item Properties: Reset Item Pitch
//Item: Select All items
//Track: set selected track grouping parameters
//
//Enveloppe: delete all selected points
//Enveloppe: insert new point at current position
//Enveloppe: reduce number of points
//Enveloppe: select points in time selection
//Enveloppe: set default point shape to ...
//Enveloppe: set shape of selected points to ...
//Enveloppe: toggle bypass for selected enveloppe
//Enveloppe: toggle display all visible envelopes in lanes for selected tracks
//Enveloppe: toggle record arm for selected envelope
//Enveloppe: toggle show all active envelopes for selected tracks
//Enveloppe: unselect all points
//
////Automation lane: set active fader (BUG in absolute mode: -50% to %12650)
//Automation: set selected tracks automation mode to ...
//
//Mixer: Toggle Show FX inserts
//Mixer: Toggle Show FX parameters
//
//Freeze

//// BUT: Mixer: toggle docking in Docker


//! \brief Dynamic RemoteSL mapping
class AutomapRemoteSlDynamic : public AutomapRemoteSl
{
	protected:
		typedef enum {	ePRESET_MAINCONTROL = 0,
						ePRESET_TRACKEDIT,
						ePRESET_ENVEDIT,
						ePRESET_MIDIEDIT,
						ePRESET_ASSIGNABLE,
						eNBPRESETS } Preset;		//!< Automap template preset

	public:
		typedef enum {	eSUBMENU_FX = 0,
						eSUBMENU_TCPFX,
						eSUBMENU_TRACKSEND_VOL, eSUBMENU_TRACKSEND_PAN, eSUBMENU_TRACKSEND_MUTE, eSUBMENU_TRACKSEND_MODE,
						eSUBMENU_TRACKHARDSEND_VOL, eSUBMENU_TRACKHARDSEND_PAN, eSUBMENU_TRACKHARDSEND_MUTE, eSUBMENU_TRACKHARDSEND_MODE,
						eNBSUBMENUS  } Submenu;					//!< Submenu for dynamic mapping

	protected:
		int _submenu;

	public:
		AutomapRemoteSlDynamic(bool hasGlobalPages);				//!< Constructor
		virtual ~AutomapRemoteSlDynamic();							//!< Destructor

		int getSubmenu();
		void setSubmenu(int submenu);
virtual int getNbSubmenus()
{
	return eNBSUBMENUS;
}

int val2Submenu(float value);
float submenu2Val(int submenu);
virtual void getSubmenuName(int submenu, char* name);

	// AutomapDevice Compliance:

	protected:
		virtual void initGlobalControllers();
		virtual void initTrackControllers(int nbTracks);

virtual void initMainControlPage(int pageId);
virtual void initTrackEditControlPage(int pageId);
virtual void initEnvEditControlPage(int pageId);
virtual void initMidiEditControlPage(int pageId);
virtual void initAssignableControlPage(int pageId);

};



//! \brief Padre's RemoteSL mapping
class AutomapRemoteSlPadres : public AutomapRemoteSlDynamic
{
	protected:
		typedef enum {	ePRESET_PADRESPAGE = 0,
						ePRESET_MIDIEDIT,
						ePRESET_MAINCONTROL,
						ePRESET_TRACKEDIT,
						ePRESET_ENVEDIT,
						ePRESET_ASSIGNABLE,
						eNBPRESETS } Preset;		//!< Automap template preset

		//typedef enum {	ePRESET_PADRESPAGE = AutomapRemoteSlDynamic::eNBPRESETS,
		//				eNBPRESETS } Preset;		//!< Automap template preset

		typedef enum {	eSUBMENU_TCPFX = 0,
						eSUBMENU_TRACKSEND_VOL, eSUBMENU_TRACKSEND_PAN, eSUBMENU_TRACKSEND_MUTE,
						eNBSUBMENUS  } Submenu;		//!< My submenus

	public:
		AutomapRemoteSlPadres(bool hasGlobalPages);				//!< Constructor
		virtual ~AutomapRemoteSlPadres();							//!< Destructor

virtual int getNbSubmenus()
{
	return eNBSUBMENUS;
}

virtual void getSubmenuName(int submenu, char* name)
{
	switch(submenu)
	{
		case eSUBMENU_TCPFX:
			return AutomapRemoteSlDynamic::getSubmenuName(AutomapRemoteSlDynamic::eSUBMENU_TCPFX, name);
		break;
		case eSUBMENU_TRACKSEND_VOL:
			return AutomapRemoteSlDynamic::getSubmenuName(AutomapRemoteSlDynamic::eSUBMENU_TRACKSEND_VOL, name);
		break;
		case eSUBMENU_TRACKSEND_PAN:
			return AutomapRemoteSlDynamic::getSubmenuName(AutomapRemoteSlDynamic::eSUBMENU_TRACKSEND_PAN, name);
		break;
		case eSUBMENU_TRACKSEND_MUTE:
			return AutomapRemoteSlDynamic::getSubmenuName(AutomapRemoteSlDynamic::eSUBMENU_TRACKSEND_MUTE, name);
		break;
		default:
			sprintf(name, "???");
		break;
	}
}


	protected:
		virtual void initGlobalControllers();
		virtual void initPadresControlPage(int pageId);
};



#endif // __AUTOMAP_MAPPING_MODS_H__
